@@ -36,8 +36,9 @@ static const char* ecasound_parameter_help_rep =
 "     -s[:]file.ecs            load chainsetup from 'file' \n"
 "     -E \"foo1 ; foo 2\"      execute interactive commands at start\n"
 "     --server                 enable ecasound's network control mode\n"
-"     --server-tcp-port=XXX    use TCP port XXX\n"
+"     --server-tcp-port=NNN    use TCP port NNN\n"
 "     --no-server              disable the daemon mode (default)\n"
+"     --osc-udp-port=NNN       listen for OSC messages on UDP port NNN\n"
 "     --keep-running (or -K)   do not exit from batchmode\n"
 "     --help (or -h)           show this help\n"
 "     --version                print version info\n"
