@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-neteci-server.c: NetECI server implementation.
-// Copyright (C) 2002,2004 Kai Vehmanen
+// Copyright (C) 2002,2004,2009 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3
@@ -20,6 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 // ------------------------------------------------------------------------
 
+#include <cassert>
 #include <cstring>        /* memcpy() */
 #include <iostream>
 #include <string>
@@ -38,7 +39,7 @@
 #include <kvu_numtostr.h>
 #include <kvu_utils.h>
 
-#include <eca-control.h>
+#include <eca-control-mt.h>
 #include <eca-logger.h>
 #include <eca-logger-wellformed.h>
 
@@ -69,7 +70,7 @@
 
 using namespace std;
 
-ECA_NETECI_SERVER::ECA_NETECI_SERVER(struct ecasound_state* state)
+ECA_NETECI_SERVER::ECA_NETECI_SERVER(ECASOUND_RUN_STATE* state)
   : state_repp(state),
     srvfd_rep(-1),
     server_listening_rep(false),
@@ -100,6 +101,12 @@ void* ECA_NETECI_SERVER::launch_server_thread(void* arg)
   return 0;
 }
 
+/**
+ * Starts running the NetECI server. 
+ *
+ * After calling this function, the ECA_CONTROL_MAIN object
+ * may be used at any time from the NetECI server thread.
+ */ 
 void ECA_NETECI_SERVER::run(void)
 {
   create_server_socket();
@@ -110,7 +117,7 @@ void ECA_NETECI_SERVER::run(void)
   else {
     ECA_LOG_MSG(ECA_LOGGER::info, 
 		"Unable to start NetECI server. Please check that no other program is using the TCP port "
-		+ kvu_numtostr(state_repp->daemon_port)
+		+ kvu_numtostr(state_repp->neteci_tcp_port)
 		+ ".");
   }
   close_server_socket();
@@ -143,7 +150,7 @@ void ECA_NETECI_SERVER::create_server_socket(void)
     srvfd_rep = socket(PF_INET, SOCK_STREAM, 0);
     if (srvfd_rep >= 0) {
       addr_in_rep.sin_family = AF_INET;
-      addr_in_rep.sin_port = htons(state_repp->daemon_port);
+      addr_in_rep.sin_port = htons(state_repp->neteci_tcp_port);
       addr_in_rep.sin_addr.s_addr = INADDR_ANY;
       
       addr_repp = reinterpret_cast<struct sockaddr*>(&addr_in_rep);
@@ -457,22 +464,20 @@ void ECA_NETECI_SERVER::parse_raw_incoming_data(const char* buffer,
 
 void ECA_NETECI_SERVER::handle_eci_command(const string& cmd, struct ecasound_neteci_server_client* client)
 {
-  ECA_CONTROL* ctrl = state_repp->control;
+  ECA_CONTROL_MT* ctrl = state_repp->control;
 
   NETECI_DEBUG(cerr << "handle eci command: " << cmd << endl);
 
-  int res = pthread_mutex_lock(state_repp->lock);
-  DBC_CHECK(res == 0);
+  assert(ctrl != 0);
 
-  DBC_CHECK(ctrl != 0);
-  ctrl->command(cmd);
+  struct eci_return_value retval;
+  ctrl->command(cmd, &retval);
 
   string strtosend =
     ECA_LOGGER_WELLFORMED::create_wellformed_message(ECA_LOGGER::eiam_return_values,
-						     ctrl->last_type() + " " + ctrl->last_value_to_string());
-
-  res = pthread_mutex_unlock(state_repp->lock);
-  DBC_CHECK(res == 0);
+      std::string(ECA_CONTROL_MAIN::return_value_type_to_string(&retval))
+      + " " + 
+      ECA_CONTROL_MAIN::return_value_to_string(&retval));
 
   int bytes_to_send = strtosend.size();
   while(bytes_to_send > 0) {
