@@ -1,6 +1,7 @@
 // ------------------------------------------------------------------------
 // audiogate.cpp: Signal gates.
-// Copyright (C) 1999-2002,2005-2007 Kai Vehmanen
+// Copyright (C) 1999-2002,2005-2008 Kai Vehmanen
+// Copyrtigh (C) 2008 Andrew Lees
 //
 // Attributes:
 //     eca-style-version: 3
@@ -41,6 +42,7 @@ void GATE_BASE::process(void)
 
 void GATE_BASE::init(SAMPLE_BUFFER* sbuf)
 { 
+  gate_open = false;
   target = sbuf;
 }
 
@@ -71,7 +73,7 @@ TIME_CROP_GATE::TIME_CROP_GATE (CHAIN_OPERATOR::parameter_t open_at, CHAIN_OPERA
   durtime_rep = duration;
   position_in_samples_rep = 0;
   
-  ECA_LOG_MSG(ECA_LOGGER::info, "(audiogate) Time crop gate created; opens at " +
+  ECA_LOG_MSG(ECA_LOGGER::info, "Time crop gate created; opens at " +
 	      kvu_numtostr(begtime_rep) + " seconds and stays open for " +
 	      kvu_numtostr(durtime_rep) + " seconds.\n");
 }
@@ -117,21 +119,30 @@ THRESHOLD_GATE::THRESHOLD_GATE (CHAIN_OPERATOR::parameter_t threshold_openlevel,
   openlevel_rep = threshold_openlevel / 100.0;
   closelevel_rep = threshold_closelevel / 100.0;
   rms_rep = use_rms;
+  reopen_count_param_rep = 0;
 
   is_opened_rep = is_closed_rep = false;
 
   if (rms_rep) {
-    ECA_LOG_MSG(ECA_LOGGER::info, "(audiogate) Threshold gate created; open threshold " +
+    ECA_LOG_MSG(ECA_LOGGER::info, "Threshold gate created; open threshold " +
 		kvu_numtostr(openlevel_rep * 100) + "%, close threshold " +
 		kvu_numtostr(closelevel_rep * 100) + "%, using RMS volume.");
   }
   else {
-    ECA_LOG_MSG(ECA_LOGGER::info, "(audiogate) Threshold gate created; open threshold " +
+    ECA_LOG_MSG(ECA_LOGGER::info, "Threshold gate created; open threshold " +
 		kvu_numtostr(openlevel_rep * 100) + "%, close threshold " +
 		kvu_numtostr(closelevel_rep * 100) + "%, using peak volume.");
   }
 }
 
+void THRESHOLD_GATE::init(SAMPLE_BUFFER* sbuf)
+{
+  reopens_left_rep = reopen_count_param_rep;
+  is_opened_rep = false;
+  is_closed_rep = false;
+  GATE_BASE::init(sbuf);
+}
+
 void THRESHOLD_GATE::analyze(SAMPLE_BUFFER* sbuf)
 {
   if (rms_rep == true)
@@ -142,15 +153,27 @@ void THRESHOLD_GATE::analyze(SAMPLE_BUFFER* sbuf)
   if (is_opened_rep == false) {
     if (avolume_rep > openlevel_rep) { 
       open_gate();
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Threshold gate opened.");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Threshold gate opened (reopen count = " + kvu_numtostr(reopens_left_rep) + ")");
       is_opened_rep = true;
+      is_closed_rep = false;
     }
   }
   else if (is_closed_rep == false) {
     if (avolume_rep < closelevel_rep) { 
       close_gate();
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Threshold gate closed.");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Threshold gate closed (reopens left = " + kvu_numtostr(reopens_left_rep) + ")");
       is_closed_rep = true;
+      if (reopens_left_rep != 0) {
+        is_opened_rep = false;
+        if (reopens_left_rep > 0)
+	  --reopens_left_rep;
+      } else {
+        // - Could we stop the engine and exit here, maybe? -AL/2008-Jul
+	// - Not from a chain operator, but the audio object
+	//   that writes the stream to a file could in
+	//   theory react in a special way to the 0-length 
+	//   samplebuffers we generate when the gate is closed... -KV/2008-Jul
+      }
     }
   }
 }
@@ -167,6 +190,8 @@ CHAIN_OPERATOR::parameter_t THRESHOLD_GATE::get_parameter(int param) const
       return 1.0;
     else 
       return 0.0;
+  case 4:
+    return reopen_count_param_rep;
   }
   return 0.0;
 }
@@ -181,10 +206,10 @@ void THRESHOLD_GATE::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
     closelevel_rep = value / 100.0;
     break;
   case 3: 
-    if (value != 0) 
-      rms_rep = true;
-    else 
-      rms_rep = false;
+    rms_rep = (value != 0);
+    break;
+  case 4:
+    reopen_count_param_rep = static_cast<int>(value);
     break;
   }
 }
