@@ -37,7 +37,7 @@
 using std::string;
 
 LOOP_DEVICE::LOOP_DEVICE(string tag) 
-  :  AUDIO_IO("loop," + tag, io_readwrite),
+  :  AUDIO_IO("loop", io_readwrite),
      tag_rep(tag),
      sbuf(buffersize(), 0)
 { 
@@ -73,37 +73,41 @@ bool LOOP_DEVICE::finished(void) const
 SAMPLE_SPECS::sample_pos_t LOOP_DEVICE::seek_position(SAMPLE_SPECS::sample_pos_t pos)
 {
   writes_rep = 0;
-  filled_rep = 0;
   empty_rounds_rep = 0;
+  filled_rep = false;
+  finished_rep = false;
   return pos;
 } 
 
 void LOOP_DEVICE::read_buffer(SAMPLE_BUFFER* buffer)
 {
-  buffer->number_of_channels(channels());
   if (empty_rounds_rep == 0) {
     if (filled_rep == true) {
-      buffer->copy(sbuf);
-      DBC_CHECK(writes_rep == registered_outputs_rep);
-      writes_rep = 0;
+      buffer->copy_all_content(sbuf);
+      /* note: read_buffer() should never be called in the middle
+       *       of the 'X * write_buffer()' sequence (where X is the number
+       *       of chain outputs connected to this loop device */
+      DBC_CHECK(writes_rep == 0);
     }
     else {
+      buffer->number_of_channels(channels());
       buffer->make_silent();
     }
   }
   else {
     finished_rep = true;
+    buffer->number_of_channels(channels());
     buffer->make_silent();
   }
 
-  DBC_CHECK(buffer->number_of_channels() == channels());
+  DBC_ENSURE(buffer->number_of_channels() == channels());
 }
 
 void LOOP_DEVICE::write_buffer(SAMPLE_BUFFER* buffer)
 {
   ++writes_rep;
 
-  /* first write after an read (or reset) */
+  /* first write of a new engine iteration (and after a read) */
   if (writes_rep == 1) {
     change_position_in_samples(buffer->length_in_samples());
     extend_position();
@@ -111,16 +115,15 @@ void LOOP_DEVICE::write_buffer(SAMPLE_BUFFER* buffer)
     sbuf.make_silent();
   }
 
+  /* check if this is the last write for this engine iteration */
+  if (writes_rep == registered_outputs_rep)
+    writes_rep = 0;
+
   /* store data from 'buffer' */
   if (buffer->is_empty() != true) {
     empty_rounds_rep = 0;
     sbuf.add_with_weight(*buffer, registered_outputs_rep);
     filled_rep = true;
-
-    if (writes_rep > registered_outputs_rep) {
-      ECA_LOG_MSG(ECA_LOGGER::info, 
-		  "WARNING: Multiple writes without reads!");
-    }
   }
   /* empty 'buffer' */
   else {
@@ -135,7 +138,7 @@ void LOOP_DEVICE::set_parameter(int param,
 {
   switch (param) {
   case 1: 
-    set_label(value);
+    AUDIO_IO::set_parameter(param, value);
     break;
 
   case 2: 
@@ -148,7 +151,7 @@ string LOOP_DEVICE::get_parameter(int param) const
 {
   switch (param) {
   case 1: 
-    return label();
+    return AUDIO_IO::get_parameter(param);
 
   case 2: 
     return tag_rep;
