@@ -2,7 +2,7 @@
 // audioio-mp3.cpp: Interface for mp3 decoders and encoders that support 
 //                  input/output using standard streams. Defaults to
 //                  mpg123 and lame.
-// Copyright (C) 1999-2006 Kai Vehmanen
+// Copyright (C) 1999-2006,2008,2009 Kai Vehmanen
 // Note! Routines for parsing mp3 header information were taken from XMMS
 //       1.2.5's mpg123 plugin. Improvements to parsing logic were
 //       contributed by Julian Dobson.
@@ -54,12 +54,17 @@
 
 #include "eca-logger.h"
 
-std::string MP3FILE::default_input_cmd = "mpg123 --stereo -b 0 -q -s -k %o %f";
-std::string MP3FILE::default_output_cmd = "lame -b %B -s %S -x -S - %f";
-long int MP3FILE::default_output_default_bitrate = 128000;
+const char *default_input_cmd = "mpg123 --stereo -b 0 -q -s -k %o %f";
+const char *default_output_cmd = "lame -b %B -s %S -r --big-endian -S - %f";
+const long int default_output_bitrate = 128000;
 
-void MP3FILE::set_input_cmd(const std::string& value) { MP3FILE::default_input_cmd = value; }
-void MP3FILE::set_output_cmd(const std::string& value) { MP3FILE::default_output_cmd = value; }
+std::string MP3FILE::conf_input_cmd = std::string(default_input_cmd);
+std::string MP3FILE::conf_output_cmd = std::string(default_output_cmd);
+
+long int MP3FILE::conf_default_output_bitrate = default_output_bitrate;
+
+void MP3FILE::set_input_cmd(const std::string& value) { MP3FILE::conf_input_cmd = value; }
+void MP3FILE::set_output_cmd(const std::string& value) { MP3FILE::conf_output_cmd = value; }
 
 /***************************************************************
  * Routines for parsing mp3 header information. Taken from XMMS
@@ -323,20 +328,21 @@ static bool mpg123_detect_by_content(const char* filename, struct frame* frp)
  **************************************************************/
 
 MP3FILE::MP3FILE(const std::string& name)
+  :  finished_rep(false),
+     triggered_rep(false)
 {
   set_label(name);
-  finished_rep = false;
-  triggered_rep = false;
   filedes_rep = -1;
   filehandle_rep = 0;
   mono_input_rep = false;
   pcm_rep = 1;
-  bitrate_rep = MP3FILE::default_output_default_bitrate;
+  bitrate_rep = MP3FILE::conf_default_output_bitrate;
 }
 
 MP3FILE::~MP3FILE(void)
 {
-  clean_child(true);
+  /* see notes in stop_io() */
+  clean_child(io_mode() == io_read ? true : false);
   if (is_open() == true) {
     close();
   }
@@ -354,6 +360,27 @@ void MP3FILE::open(void) throw(AUDIO_IO::SETUP_ERROR &)
                          count and sample format */
     set_channels(2);
     set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_le);
+
+    /* note: 'lame' command-line syntax, and default related to them, 
+     *       have changed slightly in lame 3.98, so we need this hack 
+     *       to support both old and new versions. In the past,
+     *       Ecasound wrote little-endian samples and used lame
+     *       option "-x". Newer lame versions (3.97) introduced
+     *       "--litle-endian" and "--big-endian", but these were 
+     *       buggy still in 3.97 (fixed in 3.98). And with 3.98, 
+     *       additional options (e.g. "-r") need to be passed, or
+     *       otherwise lame will exit with an error.
+     * 
+     *       In addition to above problems, we also need to remember
+     *       people updating to a newer Ecasound, but who do not update
+     *       their custom 'lame' launch commands in
+     *       ~/.ecasound/ecasoundrc (ecasound must continue to output
+     *       little-endian samples by default).
+     */
+    if (MP3FILE::conf_output_cmd.find("lame ") != std::string::npos &&
+	MP3FILE::conf_output_cmd.find(" --big-endian ") != std::string::npos) {
+      set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_be);
+    }
   }
 
   triggered_rep = false;
@@ -383,7 +410,8 @@ void MP3FILE::process_mono_fix(char* target_buffer, long int bytes) {
 
 long int MP3FILE::read_samples(void* target_buffer, long int samples)
 {
-  if (triggered_rep != true) { 
+  if (triggered_rep != true) {
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: triggering an external program in real-time context"); 
     triggered_rep = true;
     fork_input_process();
   }
@@ -391,7 +419,7 @@ long int MP3FILE::read_samples(void* target_buffer, long int samples)
   bytes_rep = std::fread(target_buffer, 1, frame_size() * samples, filehandle_rep);
   if (bytes_rep < samples * frame_size() || bytes_rep == 0) {
     if (position_in_samples() == 0) 
-      ECA_LOG_MSG(ECA_LOGGER::errors, "Can't start process \"" + MP3FILE::default_input_cmd + "\". Please check your ~/.ecasound/ecasoundrc.");
+      ECA_LOG_MSG(ECA_LOGGER::errors, "Can't start process \"" + MP3FILE::conf_input_cmd + "\". Please check your ~/.ecasound/ecasoundrc.");
     finished_rep = true;
     triggered_rep = false;
   }
@@ -413,29 +441,41 @@ void MP3FILE::write_samples(void* target_buffer, long int samples)
   if (wait_for_child() != true) {
     finished_rep = true;
     triggered_rep = false;
+    ECA_LOG_MSG(ECA_LOGGER::errors, "Attempt to write after child process has terminated.");
   }
   else {
     bytes_rep = ::write(filedes_rep, target_buffer, frame_size() * samples);
-    if (bytes_rep < frame_size() * samples || bytes_rep == 0) {
+
+    if (bytes_rep < frame_size() * samples) {
       if (position_in_samples() == 0) 
-	ECA_LOG_MSG(ECA_LOGGER::errors, "Can't start process \"" + MP3FILE::default_output_cmd + "\". Please check your ~/.ecasound/ecasoundrc.");
+	ECA_LOG_MSG(ECA_LOGGER::errors, "Can't start process \"" + MP3FILE::conf_output_cmd + "\". Please check your ~/.ecasound/ecasoundrc.");
+      else
+	ECA_LOG_MSG(ECA_LOGGER::errors, 
+		    "Error in writing to child process (to write " 
+		    + kvu_numtostr(frame_size() * samples) 
+		    + ", result "
+		    + kvu_numtostr(bytes_rep) 
+		    + ").");
+
       finished_rep = true;
     }
-    else finished_rep = false;
+    else 
+      finished_rep = false;
   }
 }
 
-void MP3FILE::seek_position(void)
+SAMPLE_SPECS::sample_pos_t MP3FILE::seek_position(SAMPLE_SPECS::sample_pos_t pos)
 {
   finished_rep = false;
   if (triggered_rep == true &&
-      last_position_rep != position_in_samples()) {
+      last_position_rep != pos) {
     if (is_open() == true) {
       ECA_LOG_MSG(ECA_LOGGER::user_objects, "Cleaning child process pid=" + kvu_numtostr(pid_of_child()) + ".");
-      clean_child();
+      clean_child(true);
       triggered_rep = false;
     }
   }
+  return pos;
 }
 
 void MP3FILE::set_parameter(int param, std::string value)
@@ -450,7 +490,7 @@ void MP3FILE::set_parameter(int param, std::string value)
     if (numvalue > 0) 
       bitrate_rep = numvalue;
     else
-      bitrate_rep = MP3FILE::default_output_default_bitrate;
+      bitrate_rep = MP3FILE::conf_default_output_bitrate;
     break;
   }
 }
@@ -521,9 +561,37 @@ void MP3FILE::get_mp3_params(const std::string& fname) throw(AUDIO_IO::SETUP_ERR
   set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_le);
 }
 
+void MP3FILE::start_io(void)
+{
+  if (triggered_rep != true) {
+    if (io_mode() == io_read) 
+      fork_input_process();
+    else
+      fork_output_process();
+
+    triggered_rep = true;
+  }
+}
+
+void MP3FILE::stop_io(void)
+{
+  if (triggered_rep == true) {
+    /* note: it's safe to send a SIGTERM if the client is 
+     *       an input and we know its PID (otherwise 
+     *       cleanup will still work but will take more time, which
+     *       is nasty if we are in a middle of a seek */
+    if (io_mode() == io_read) 
+      clean_child(true);
+    else
+      clean_child(false);
+
+    triggered_rep = false;
+  }
+}
+
 void MP3FILE::fork_input_process(void)
 {
-  std::string cmd = MP3FILE::default_input_cmd;
+  std::string cmd = MP3FILE::conf_input_cmd;
   if (cmd.find("%o") != std::string::npos) {
     cmd.replace(cmd.find("%o"), 2, kvu_numtostr((long)(position_in_samples() / pcm_rep)));
   }
@@ -554,7 +622,7 @@ void MP3FILE::fork_output_process(void)
 {
   ECA_LOG_MSG(ECA_LOGGER::info, "Starting to encode " + label() + " with lame.");
   last_position_rep = position_in_samples();
-  std::string cmd = MP3FILE::default_output_cmd;
+  std::string cmd = MP3FILE::conf_output_cmd;
   if (cmd.find("%B") != std::string::npos) {
     cmd.replace(cmd.find("%B"), 2, kvu_numtostr((long int)(bitrate_rep / 1000)));
   }
