@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-chainsetup.cpp: Class representing an ecasound chainsetup object.
-// Copyright (C) 1999-2006,2008 Kai Vehmanen
+// Copyright (C) 1999-2006,2008,2009 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
@@ -83,9 +83,10 @@
 
 using std::cerr;
 using std::endl;
+using namespace ECA;
 
 const string ECA_CHAINSETUP::default_audio_format_const = "s16_le,2,44100,i";
-const string ECA_CHAINSETUP::default_bmode_nonrt_const = "1024,true,50,true,100000,true";
+const string ECA_CHAINSETUP::default_bmode_nonrt_const = "1024,false,50,false,100000,true";
 const string ECA_CHAINSETUP::default_bmode_rt_const = "1024,true,50,true,100000,true";
 const string ECA_CHAINSETUP::default_bmode_rtlowlatency_const = "256,true,50,true,100000,false";
 
@@ -103,14 +104,11 @@ ECA_CHAINSETUP::ECA_CHAINSETUP(const vector<string>& opts)
   : cparser_rep(this),
     is_enabled_rep(false)
 {
-
-  ECA_LOG_MSG(ECA_LOGGER::subsystems, "Chainsetup created (cmdline)");
-
   impl_repp = new ECA_CHAINSETUP_impl;
 
   // FIXME: set default audio format here!
 
-  setup_name_rep = "command-line-setup";
+  setup_name_rep = "untitled-chainsetup";
   setup_filename_rep = "";
 
   set_defaults();
@@ -124,6 +122,8 @@ ECA_CHAINSETUP::ECA_CHAINSETUP(const vector<string>& opts)
     add_default_output();
     add_default_midi_device();
   }
+
+  ECA_LOG_MSG(ECA_LOGGER::info, "Chainsetup \"" + setup_name_rep + "\"");
 }
 
 /**
@@ -135,12 +135,12 @@ ECA_CHAINSETUP::ECA_CHAINSETUP(void)
   : cparser_rep(this),
     is_enabled_rep(false)
 {
-  ECA_LOG_MSG(ECA_LOGGER::subsystems, "Chainsetup created (empty)");
-
   impl_repp = new ECA_CHAINSETUP_impl;
 
   setup_name_rep = "";
   set_defaults();
+
+  ECA_LOG_MSG(ECA_LOGGER::info, "Chainsetup created (empty)");
 }
 
 /**
@@ -157,8 +157,6 @@ ECA_CHAINSETUP::ECA_CHAINSETUP(const string& setup_file)
   : cparser_rep(this),
     is_enabled_rep(false)
 {
-  ECA_LOG_MSG(ECA_LOGGER::subsystems, "Chainsetup created (file)");
-
   impl_repp = new ECA_CHAINSETUP_impl;
 
   setup_name_rep = "";
@@ -174,6 +172,11 @@ ECA_CHAINSETUP::ECA_CHAINSETUP(const string& setup_file)
      * it might hide real problems */
     add_default_output();
   }
+
+  ECA_LOG_MSG(ECA_LOGGER::info, 
+	      "Chainsetup \"" 
+	      + name() + "\" created (file: "
+	      + setup_file + ")"); 
 }
 
 /**
@@ -181,7 +184,7 @@ ECA_CHAINSETUP::ECA_CHAINSETUP(const string& setup_file)
  */
 ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
 { 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects,"ECA_CHAINSETUP destructor!");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects,"ECA_CHAINSETUP destructor-in");
 
   DBC_CHECK(is_locked() != true);
   if (is_enabled() == true) {
@@ -256,6 +259,8 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
   }
 
   delete impl_repp;
+
+  ECA_LOG_MSG(ECA_LOGGER::system_objects,"ECA_CHAINSETUP destructor-out");
 }
 
 /**
@@ -269,6 +274,8 @@ void ECA_CHAINSETUP::set_defaults(void)
   DBC_REQUIRE(is_enabled() != true);
   // --------
 
+  /* note: defaults are set as specified in ecasoundrc(5) */
+
   precise_sample_rates_rep = false;
   ignore_xruns_rep = true;
 
@@ -276,8 +283,7 @@ void ECA_CHAINSETUP::set_defaults(void)
   midi_server_repp = &impl_repp->midi_server_rep;
   engine_driver_repp = 0;
 
-  if (kvu_check_for_mlockall() == true && 
-      kvu_check_for_sched_fifo() == true) {
+  if (kvu_check_for_sched_fifo() == true) {
     rtcaps_rep = true;
     ECA_LOG_MSG(ECA_LOGGER::system_objects, "Rtcaps detected.");
   }
@@ -290,11 +296,9 @@ void ECA_CHAINSETUP::set_defaults(void)
   memory_locked_rep = false;
   midi_server_needed_rep = false;
   is_locked_rep = false;
-  active_chain_index_rep = 0;
-  active_chainop_index_rep = 0;
-  active_chainop_param_index_rep = 0;
-  active_ctrl_index_rep = 0;
-  active_ctrl_param_index_rep = 0;
+  selected_chain_index_rep = 0;
+  selected_ctrl_index_rep = 0;
+  selected_ctrl_param_index_rep = 0;
   multitrack_mode_offset_rep = -1;
 
   buffering_mode_rep = cs_bmode_auto;
@@ -345,8 +349,8 @@ string ECA_CHAINSETUP::set_resource_helper(const ECA_RESOURCES& ecaresources, co
   }
   else {
     ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		"(eca-chaisetup) Using hardcoded defaults for '" +
-		tag + "'.");
+		"Using hardcoded defaults for \"" +
+		tag + "\".");
     return alternative;
   }
 }
@@ -456,9 +460,9 @@ void ECA_CHAINSETUP::set_buffering_mode(Buffering_mode_t value)
 void ECA_CHAINSETUP::set_audio_io_manager_option(const string& mgrname, const string& optionstr)
 {
   ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-	      "Set manager '" +
-	      mgrname + "' option string to '" +
-	      optionstr + "'.");
+	      "Set manager \"" +
+	      mgrname + "\" option string to \"" +
+	      optionstr + "\".");
 
   aio_manager_option_map_rep[mgrname] = optionstr;
   propagate_audio_io_manager_options();
@@ -506,6 +510,8 @@ void ECA_CHAINSETUP::select_active_buffering_mode(void)
 
       /* case 2: rt-objects without priviledges for rt-scheduling */
       else if (rtcaps_rep != true) {
+	ECA_LOG_MSG(ECA_LOGGER::info,
+		    "NOTE: Real-time configuration, but insufficient privileges to utilize real-time scheduling (SCHED_FIFO). With small buffersizes, this may cause audible glitches during processing.");
 	toggle_raised_priority(false);
 	active_buffering_mode_rep = ECA_CHAINSETUP::cs_bmode_rt;
 	ECA_LOG_MSG(ECA_LOGGER::system_objects, "bmode-selection case-2");
@@ -542,19 +548,19 @@ void ECA_CHAINSETUP::select_active_buffering_mode(void)
     case ECA_CHAINSETUP::cs_bmode_nonrt: { 
       impl_repp->bmode_active_rep = impl_repp->bmode_nonrt_rep;
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "'nonrt' buffering mode selected.");
+		    "\"nonrt\" buffering mode selected.");
       break; 
     }
     case ECA_CHAINSETUP::cs_bmode_rt: { 
       impl_repp->bmode_active_rep = impl_repp->bmode_rt_rep;
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "'rt' buffering mode selected.");
+		    "\"rt\" buffering mode selected.");
       break; 
     }
     case ECA_CHAINSETUP::cs_bmode_rtlowlatency: { 
       impl_repp->bmode_active_rep = impl_repp->bmode_rtlowlatency_rep;
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "'rtlowlatency' buffering mode selected.");
+		    "\"rtlowlatency\" buffering mode selected.");
       break;
     }
     default: { /* error! */ }
@@ -848,7 +854,7 @@ void ECA_CHAINSETUP::select_all_chains(void)
 
 /**
  * Returns the index number of first selected chains. If no chains 
- * are selected, returns 'last_index + 1' (chains.size()).
+ * are selected, returns 'last_index + 1' (==chains.size()).
  */
 unsigned int ECA_CHAINSETUP::first_selected_chain(void) const
 {
@@ -1166,10 +1172,10 @@ AUDIO_IO_MANAGER* ECA_CHAINSETUP::get_audio_object_manager(AUDIO_IO* aio) const
     if ((*q)->is_managed_type(aio) && 
 	(*q)->get_object_id(aio) != -1) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "Found object manager '" +
+		    "Found object manager \"" +
 		    (*q)->name() + 
-		    "' for aio '" +
-		    aio->label() + "'.");
+		    "\" for aio \"" +
+		    aio->label() + "\".");
       
       return *q;
     }
@@ -1188,10 +1194,10 @@ AUDIO_IO_MANAGER* ECA_CHAINSETUP::get_audio_object_type_manager(AUDIO_IO* aio) c
   for(vector<AUDIO_IO_MANAGER*>::const_iterator q = aio_managers_rep.begin(); q != aio_managers_rep.end(); q++) {
     if ((*q)->is_managed_type(aio) == true) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "Found object manager '" +
+		    "Found object manager \"" +
 		    (*q)->name() + 
-		    "' for aio type '" +
-		    aio->name() + "'.");
+		    "\" for aio type \"" +
+		    aio->name() + "\".");
       
       return *q;
     }
@@ -1210,9 +1216,9 @@ void ECA_CHAINSETUP::register_engine_driver(AUDIO_IO_MANAGER* amgr)
   if (driver != 0) {
     engine_driver_repp = driver;
     ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		  "Registered audio i/o manager '" +
+		  "Registered audio i/o manager \"" +
 		  amgr->name() +
-		  "' as the current engine driver.");
+		  "\" as the current engine driver.");
   }
 }
 
@@ -1228,10 +1234,10 @@ void ECA_CHAINSETUP::register_audio_object_to_manager(AUDIO_IO* aio)
     mgr = aio->create_object_manager();
     if (mgr != 0) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "Creating object manager '" +
+		    "Creating object manager \"" +
 		    mgr->name() + 
-		    "' for aio '" +
-		    aio->name() + "'.");
+		    "\" for aio \"" +
+		    aio->name() + "\".");
       aio_managers_rep.push_back(mgr);
       propagate_audio_io_manager_options();
       mgr->register_object(aio);
@@ -1255,10 +1261,10 @@ void ECA_CHAINSETUP::unregister_audio_object_from_manager(AUDIO_IO* aio)
     int id = mgr->get_object_id(aio);
     if (id != -1) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "Unregistering object '" +
+		    "Unregistering object \"" +
 		    aio->name() + 
-		    "' from manager '" +
-		    mgr->name() + "'.");
+		    "\" from manager \"" +
+		    mgr->name() + "\".");
       mgr->unregister_object(id);
     }
   }
@@ -1279,10 +1285,10 @@ void ECA_CHAINSETUP::propagate_audio_io_manager_options(void)
       for(int n = 0; n < numparams; n++) {
 	(*q)->set_parameter(n + 1, kvu_get_argument_number(n + 1, optstring));
 	ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "Manager '" +
-		    (*q)->name() + "', " + 
-		    kvu_numtostr(n + 1) + ". parameter set to '" +
-		    (*q)->get_parameter(n + 1) + "'.");
+		    "Manager \"" +
+		    (*q)->name() + "\", " + 
+		    kvu_numtostr(n + 1) + ". parameter set to \"" +
+		    (*q)->get_parameter(n + 1) + "\".");
       }
     }      
   }
@@ -1579,7 +1585,7 @@ void ECA_CHAINSETUP::audio_object_open_info(const AUDIO_IO* aio)
   DBC_REQUIRE(aio != 0);
   // --------
 
-  string temp = "Opening ";
+  string temp = "Opened ";
   
   temp += (aio->io_mode() == AUDIO_IO::io_read) ? "input" : "output";
   temp += " \"" + aio->label();
@@ -1646,6 +1652,31 @@ const CHAIN* ECA_CHAINSETUP::get_chain_with_name(const string& name) const
   return 0;
 }
 
+/**
+ * Returns a non-zero index for chain 'name'. If the chain
+ * does not exist, -1 is returned.
+ * 
+ * The chain index can be used with ECA::chainsetup_edit_t 
+ * items passed to ECA_CHAINSETUP::execute_edit().
+ *
+ * Note: Mapping of chain names to indices can change if any
+ *       chains are either added or removed. If that happens,
+ *       the indices need to be recalculated. 
+ */
+int ECA_CHAINSETUP::get_chain_index(const string& name) const
+{
+  int retval = -1;
+  vector<CHAIN*>::const_iterator p = chains.begin();
+  for(int n = 1; p != chains.end(); n++) {
+    if ((*p)->name() == name) {
+      retval = n;
+      break;
+    }
+    ++p;
+  }
+  return retval;
+}
+
 /**
  * Attaches input 'obj' to all selected chains.
  *
@@ -1747,9 +1778,9 @@ void ECA_CHAINSETUP::check_object_samplerate(const AUDIO_IO* obj,
   if (obj->samples_per_second() != srate) {
     throw(ECA_ERROR("ECA-CHAINSETUP", 
 		    string("All audio objects must have a common") +
-		    " sampling rate; sampling rate of audio object '" +
+		    " sampling rate; sampling rate of audio object \"" +
  		    obj->label() +
-		    "' differs from engine rate (" +
+		    "\" differs from engine rate (" +
 		    kvu_numtostr(obj->samples_per_second()) +
 		    " <-> " + 
 		    kvu_numtostr(srate) + 
@@ -1958,6 +1989,68 @@ void ECA_CHAINSETUP::disable(void)
   // --------
 }
 
+/**
+ * Executes chainsetup edit 'edit'.
+ *
+ * @return true if succesful, false if edit cannot
+ *         be performed
+ */
+bool ECA_CHAINSETUP::execute_edit(const chainsetup_edit_t& edit)
+{
+  bool retval = true;
+
+  ECA_LOG_MSG(ECA_LOGGER::user_objects,
+	      "Executing edit type of " +
+	      kvu_numtostr(static_cast<int>(edit.type)));
+
+  if (edit.cs_ptr != this) {
+    ECA_LOG_MSG(ECA_LOGGER::errors, 
+		"ERROR: chainsetup edit executed on wrong object");
+    return false;
+  }
+
+  switch(edit.type)
+    {
+    case edit_cop_set_param:
+      {
+	if (edit.m.cop_set_param.chain < 1 ||
+	    edit.m.cop_set_param.chain > static_cast<int>(chains.size())) {
+	  retval = false;
+	  break;
+	}
+	CHAIN *ch = chains[edit.m.cop_set_param.chain - 1];
+	ch->set_parameter(edit.m.cop_set_param.op, 
+			  edit.m.cop_set_param.param,
+			  edit.m.cop_set_param.value);
+	break;
+      }
+    case edit_ctrl_set_param:
+      {
+	if (edit.m.ctrl_set_param.chain < 1 ||
+	    edit.m.ctrl_set_param.chain > static_cast<int>(chains.size())) {
+	  retval = false;
+	  break;
+	}
+	CHAIN *ch = chains[edit.m.ctrl_set_param.chain - 1];
+	ch->set_controller_parameter(edit.m.ctrl_set_param.op,
+				     edit.m.ctrl_set_param.param,
+				     edit.m.ctrl_set_param.value);
+	break;
+      }
+    default:
+      {
+	DBC_NEVER_REACHED();
+	retval = false;
+	ECA_LOG_MSG(ECA_LOGGER::info,
+		    "Unknown edit of type " +
+		    kvu_numtostr(static_cast<int>(edit.type)));
+	break;
+      }
+    }
+
+  return retval;
+}
+
 /**
  * Updates the chainsetup processing length based on 
  * 1) requested length, 2) lengths of individual 
@@ -2000,25 +2093,13 @@ long int ECA_CHAINSETUP::check_for_locked_buffersize(void) const
 #ifdef ECA_COMPILE_JACK
   int pid = getpid();
   string cname = "ecasound-ctrl-" + kvu_numtostr(pid);
-  int jackobjs = 0, resampleobjs = 0;
+  int jackobjs = 0;
 
   for(size_t n = 0; n < inputs_direct_rep.size(); n++) {
-    AUDIO_IO_RESAMPLE *p = dynamic_cast<AUDIO_IO_RESAMPLE*>(inputs_direct_rep[n]);
-    if (p != 0) ++resampleobjs;
     if (inputs_direct_rep[n]->name() == "JACK interface") ++jackobjs;
-    if (resampleobjs && jackobjs) break;
   }
   for(size_t n = 0; n < outputs_direct_rep.size(); n++) {
-    AUDIO_IO_RESAMPLE *p = dynamic_cast<AUDIO_IO_RESAMPLE*>(outputs_direct_rep[n]);
-    if (p != 0) ++resampleobjs;
     if (outputs_direct_rep[n]->name() == "JACK interface") ++jackobjs;
-    if (resampleobjs && jackobjs) break;
-  }
-
-  if (resampleobjs && jackobjs) {
-    ECA_LOG_MSG(ECA_LOGGER::info, 
-		"WARNING: Mixing resample and jack objects may result garbled audio. "
-		"Problem occurs when buffersize is not an integer multiple of resample ratio.");
   }
 
   /* contact jackd only if there is at least one jack audio object 
@@ -2088,10 +2169,11 @@ static void priv_seek_position_helper(std::vector<AUDIO_IO*>* objs, SAMPLE_SPECS
       /* note: report if object claims it supports seeking, but
        *       in fact the seek failed */
       if ((*q)->supports_seeking() == true) {
-	if ((*q)->position_in_samples() != pos)
+	if (pos <= (*q)->length_in_samples() &&
+	    (*q)->position_in_samples() != pos)
 	  ECA_LOG_MSG(ECA_LOGGER::info,
 		      "WARNING: sample accurate seek failed with " +
-		      tag + " '" + (*q)->name() + "'");
+		      tag + " \"" + (*q)->name() + "\"");
       }
     }
   }
@@ -2103,9 +2185,9 @@ static void priv_seek_position_helper(std::vector<AUDIO_IO*>* objs, SAMPLE_SPECS
 SAMPLE_SPECS::sample_pos_t ECA_CHAINSETUP::seek_position(SAMPLE_SPECS::sample_pos_t pos)
 {
   ECA_LOG_MSG(ECA_LOGGER::user_objects,
-	      "seek position, chainsetup '" +
+	      "seek position, chainsetup \"" +
 	      name() +
-	      "' to pos in sec " + 
+	      "\" to pos in sec " + 
 	      kvu_numtostr(pos) + ".");
 
   if (is_enabled() == true) {
@@ -2119,8 +2201,8 @@ SAMPLE_SPECS::sample_pos_t ECA_CHAINSETUP::seek_position(SAMPLE_SPECS::sample_po
     (*q)->seek_position_in_samples(pos);
     if ((*q)->position_in_samples() != pos)
       ECA_LOG_MSG(ECA_LOGGER::info,
-		  "WARNING: sample accurate seek failed with chainop '" +
-		  (*q)->name() + "'");
+		  "WARNING: sample accurate seek failed with chainop \"" +
+		  (*q)->name() + "\"");
     
   }
 
