@@ -30,6 +30,7 @@
 
 #include "eca-chainsetup-position.h"
 #include "eca-chainsetup-parser.h"
+#include "eca-chainsetup-edit.h"
 #include "eca-error.h"
 
 class AUDIO_IO;
@@ -78,6 +79,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
   friend class ECA_CONTROL_OBJECTS;
   friend class ECA_CHAINSETUP_PARSER;
 
+  // -------------------------------------------------------------------
+
   /** @name Public type definitions and constants */
   /*@{*/
 
@@ -96,6 +99,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for init and cleanup */
   /*@{*/
 
@@ -106,6 +111,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for handling audio objects */
   /*@{*/
 
@@ -123,6 +130,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for handling chains */
   /*@{*/
 
@@ -141,9 +150,12 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
   vector<string> chain_names(void) const;
   vector<string> get_attached_chains_to_iodev(const string& filename) const;
   const CHAIN* get_chain_with_name(const string& name) const;
+  int get_chain_index(const string& name) const;
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for handling MIDI-objects */
   /*@{*/
 
@@ -153,6 +165,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for chain operators */
   /*@{*/
 
@@ -162,6 +176,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for configuration (default values, settings) */
   /*@{*/
 
@@ -187,6 +203,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for overriding current buffering mode parameters */
   /*@{*/
 
@@ -206,6 +224,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions that modify current state  */
   /*@{*/
 
@@ -217,8 +237,12 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
   const string& name(void) const { return setup_name_rep; }
   const string& filename(void) const { return setup_filename_rep; }
 
+  bool execute_edit(const ECA::chainsetup_edit_t& edit);
+
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions implemented from ECA_SAMPLERATE_AWARE */
   /*@{*/
 
@@ -226,6 +250,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions implemented from ECA_AUDIO_POSITION */
   /*@{*/
 
@@ -233,6 +259,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for observing current state */
   /*@{*/
 
@@ -255,6 +283,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for string->state conversions */
   /*@{*/
 
@@ -274,16 +304,20 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for string<->state conversions */
   /*@{*/
 
   void save(void) throw(ECA_ERROR&);
   void save_to_file(const string& filename) throw(ECA_ERROR&);
 
- private:
-
   /*@}*/
 
+  // -------------------------------------------------------------------
+
+ private:
+
   /** @name Configuration data (settings and values)  */
   /*@{*/
   
@@ -299,6 +333,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Current setup data (internal state, objects) */
   /*@{*/
 
@@ -308,11 +344,14 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
   bool multitrack_mode_override_rep;
   bool memory_locked_rep;
   bool midi_server_needed_rep;
-  int active_chain_index_rep;
-  int active_chainop_index_rep;
-  int active_chainop_param_index_rep;
-  int active_ctrl_index_rep;
-  int active_ctrl_param_index_rep;
+
+  /* FIXME: only needed by ECA_ENGINE */
+  int selected_chain_index_rep;
+  int selected_cop_index_rep;
+  int selected_cop_param_index_rep;
+  int selected_ctrl_index_rep;
+  int selected_ctrl_param_index_rep;
+
   int db_clients_rep;
   long int multitrack_mode_offset_rep;
   string setup_name_rep;
@@ -343,6 +382,7 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
   ECA_ENGINE_DRIVER* engine_driver_repp;
 
   /*@}*/
+  // -------------------------------------------------------------------
 
   /** @name Functions for handling audio objects */
   /*@{*/
@@ -358,6 +398,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
   void remove_audio_object_loop(const string& label, AUDIO_IO* aio, int dir);
   void remove_audio_object_impl(const string& label, int dir, bool destroy);
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for state<->string conversions */
   /*@{*/
 
@@ -365,6 +407,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Functions for internal state changes */
   /*@{*/
 
@@ -387,6 +431,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Private helper functions */
   /*@{*/
 
@@ -402,6 +448,8 @@ class ECA_CHAINSETUP : public ECA_CHAINSETUP_POSITION {
 
   /*@}*/
 
+  // -------------------------------------------------------------------
+
   /** @name Static private helper functions */
   /*@{*/
 
