@@ -1,7 +1,7 @@
 // ------------------------------------------------------------------------
 // eca-control-base.cpp: Base class providing basic functionality
 //                       for controlling the ecasound library
-// Copyright (C) 1999-2004,2006,2008 Kai Vehmanen
+// Copyright (C) 1999-2004,2006,2008,2009 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3 (see Ecasound Programmer's Guide)
@@ -37,8 +37,8 @@
 #include "eca-engine.h"
 #include "eca-session.h"
 #include "eca-chainsetup.h"
-#include "eca-control-base.h"
 #include "eca-resources.h"
+#include "eca-control.h"
 
 #include "eca-error.h"
 #include "eca-logger.h"
@@ -57,9 +57,9 @@ using std::vector;
 /**
  * Helper function for starting the slave thread.
  */
-void* ECA_CONTROL_BASE::start_normal_thread(void *ptr)
+void* ECA_CONTROL::start_normal_thread(void *ptr)
 {
-  ECA_CONTROL_BASE* ctrl_base = static_cast<ECA_CONTROL_BASE*>(ptr);
+  ECA_CONTROL* ctrl_base = static_cast<ECA_CONTROL*>(ptr);
   ctrl_base->engine_pid_rep = getpid();
   DBC_CHECK(ctrl_base->engine_pid_rep >= 0);
 
@@ -74,33 +74,17 @@ void* ECA_CONTROL_BASE::start_normal_thread(void *ptr)
   return 0;
 }
 
-ECA_CONTROL_BASE::ECA_CONTROL_BASE (ECA_SESSION* psession)
-{
-  session_repp = psession;
-  selected_chainsetup_repp = psession->selected_chainsetup_repp;
-  engine_repp = 0;
-  engine_pid_rep = -1;
-  engine_exited_rep.set(0);
-  float_to_string_precision_rep = 3;
-  joining_rep = false;
-}
-
-ECA_CONTROL_BASE::~ECA_CONTROL_BASE (void)
-{
-  close_engine();
-}
-
 /**
  * Initializes the engine
  *
  * @pre is_connected() == true
- * @pre is_engine_started() != true
+ * @pre is_engine_created() != true
  */
-void ECA_CONTROL_BASE::engine_start(void)
+void ECA_CONTROL::engine_start(void)
 {
   // --------
   DBC_REQUIRE(is_connected() == true);
-  DBC_REQUIRE(is_engine_started() != true);
+  DBC_REQUIRE(is_engine_created() != true);
   // --------
 
   start_engine_sub(false);
@@ -111,11 +95,11 @@ void ECA_CONTROL_BASE::engine_start(void)
  *
  * @pre is_connected() == true
  * @pre is_running() != true
- * @post is_engine_started() == true
+ * @post is_engine_created() == true
  *
  * @return negative on error, zero on success 
  */
-int ECA_CONTROL_BASE::start(void)
+int ECA_CONTROL::start(void)
 {
   // --------
   DBC_REQUIRE(is_connected() == true);
@@ -126,12 +110,12 @@ int ECA_CONTROL_BASE::start(void)
 
   ECA_LOG_MSG(ECA_LOGGER::subsystems, "Controller/Processing started");
 
-  if (is_engine_started() != true) {
+  if (is_engine_created() != true) {
     /* request_batchmode=false */
     start_engine_sub(false);
   }
 
-  if (is_engine_started() != true) {
+  if (is_engine_created() != true) {
     ECA_LOG_MSG(ECA_LOGGER::info, "Can't start processing: couldn't start engine.");
     result = -1;
   }  
@@ -140,7 +124,7 @@ int ECA_CONTROL_BASE::start(void)
   }
 
   // --------
-  DBC_ENSURE(result != 0 || is_engine_started() == true);
+  DBC_ENSURE(result != 0 || is_engine_created() == true);
   // --------
 
   return result;
@@ -158,13 +142,13 @@ int ECA_CONTROL_BASE::start(void)
  * @post is_finished() == true || 
  *       processing_started == true && is_running() != true ||
  *       processing_started != true &&
- *       (is_engine_started() != true ||
- *        is_engine_started() == true &&
+ *       (is_engine_created() != true ||
+ *        is_engine_created() == true &&
  * 	  engine_repp->status() != ECA_ENGINE::engine_status_stopped))
  *
  * @return negative on error, zero on success 
  */
-int ECA_CONTROL_BASE::run(bool batchmode)
+int ECA_CONTROL::run(bool batchmode)
 {
   // --------
   DBC_REQUIRE(is_connected() == true);
@@ -176,12 +160,12 @@ int ECA_CONTROL_BASE::run(bool batchmode)
   bool processing_started = false;
   int result = -1;
 
-  if (is_engine_started() != true) {
+  if (is_engine_created() != true) {
     /* request_batchmode=true */
     start_engine_sub(batchmode);
   }
 
-  if (is_engine_started() != true) {
+  if (is_engine_created() != true) {
     ECA_LOG_MSG(ECA_LOGGER::info, "Can't start processing: couldn't start the engine. (2)");
   } 
   else { 
@@ -195,8 +179,8 @@ int ECA_CONTROL_BASE::run(bool batchmode)
      * until error occurs) */
     while(is_finished() != true || batchmode != true) {
       
-      /* sleep for one second */
-      kvu_sleep(1, 0);
+      /* sleep for 250ms */
+      kvu_sleep(0, 250000000);
 
       if (processing_started != true) {
 	if (is_running() == true ||
@@ -205,7 +189,7 @@ int ECA_CONTROL_BASE::run(bool batchmode)
 	  /* make a note that engine state changed to 'running' */
 	  processing_started = true;
 	}
-	else if (is_engine_started() == true) {
+	else if (is_engine_created() == true) {
 	  if (engine_repp->status() == ECA_ENGINE::engine_status_error) {
 	    /* not running, so status() is either 'not_ready' or 'error' */
 	    ECA_LOG_MSG(ECA_LOGGER::info, "Can't start processing: engine startup failed. (3)");
@@ -218,7 +202,7 @@ int ECA_CONTROL_BASE::run(bool batchmode)
 	}
 	else {
 	  /* ECA_CONTROL_BASE destructor has been run and 
-	   * engine_repp is now 0 (--> is_engine_started() != true) */
+	   * engine_repp is now 0 (--> is_engine_created() != true) */
 	  break;
 	}
       }
@@ -241,13 +225,13 @@ int ECA_CONTROL_BASE::run(bool batchmode)
   ECA_LOG_MSG(ECA_LOGGER::subsystems, 
 	      std::string("Controller/Batch processing finished (")
 	      + kvu_numtostr(result) + ")");
-  
+
   // --------
   DBC_ENSURE(is_finished() == true ||
 	     (processing_started == true && is_running()) != true ||
 	     (processing_started != true &&
-	      (is_engine_started() != true ||
-	       (is_engine_started() == true &&
+	      (is_engine_created() != true ||
+	       (is_engine_created() == true &&
 		engine_repp->status() != ECA_ENGINE::engine_status_stopped))));
   // --------
 
@@ -257,14 +241,16 @@ int ECA_CONTROL_BASE::run(bool batchmode)
 /**
  * Stops the processing engine.
  *
- * @pre is_engine_started() == true
+ * @see stop_on_condition()
+ *
+ * @pre is_engine_created() == true
  * @pre is_running() == true
  * @post is_running() == false
  */
-void ECA_CONTROL_BASE::stop(void)
+void ECA_CONTROL::stop(void)
 {
   // --------
-  DBC_REQUIRE(is_engine_started() == true);
+  DBC_REQUIRE(is_engine_created() == true);
   DBC_REQUIRE(is_running() == true);
   // --------
 
@@ -282,13 +268,13 @@ void ECA_CONTROL_BASE::stop(void)
  * Stop the processing engine using thread-to-thread condition
  * signaling.
  *
- * @pre is_engine_started() == true
+ * @pre is_engine_created() == true
  * @post is_running() == false
  */
-void ECA_CONTROL_BASE::stop_on_condition(void)
+void ECA_CONTROL::stop_on_condition(void)
 {
   // --------
-  DBC_REQUIRE(is_engine_started() == true);
+  DBC_REQUIRE(is_engine_created() == true);
   // --------
 
   if (engine_repp->status() != ECA_ENGINE::engine_status_running) return;
@@ -309,16 +295,17 @@ void ECA_CONTROL_BASE::stop_on_condition(void)
  * Stops the processing engine.
  * Call will block until engine has terminated.
  */
-void ECA_CONTROL_BASE::quit(void) { close_engine(); }
+void ECA_CONTROL::quit(void) { close_engine(); }
 
 /**
  * Stops the processing engine. A thread-safe variant of
  * quit(). Call will not block.
  *
  */
-void ECA_CONTROL_BASE::quit_async(void)
+void ECA_CONTROL::quit_async(void)
 {
-  if (is_engine_started() != true) return;
+  if (is_engine_running() != true) 
+    return;
 
   engine_repp->command(ECA_ENGINE::ep_exit, 0.0);
 }
@@ -327,22 +314,27 @@ void ECA_CONTROL_BASE::quit_async(void)
  * Starts the processing engine.
  *
  * @pre is_connected() == true
- * @pre is_engine_started() != true
+ * @pre is_engine_running() != true
  */
-void ECA_CONTROL_BASE::start_engine_sub(bool batchmode)
+void ECA_CONTROL::start_engine_sub(bool batchmode)
 {
   // --------
   DBC_REQUIRE(is_connected() == true);
-  DBC_REQUIRE(is_engine_started() != true);
+  DBC_REQUIRE(is_engine_running() != true);
   // --------
 
   DBC_CHECK(engine_exited_rep.get() != 1);
 
   unsigned int p = session_repp->connected_chainsetup_repp->first_selected_chain();
   if (p < session_repp->connected_chainsetup_repp->chains.size())
-    session_repp->connected_chainsetup_repp->active_chain_index_rep = p;
+    session_repp->connected_chainsetup_repp->selected_chain_index_rep = p;
+  
+  if (engine_repp)
+    close_engine();
 
+  DBC_CHECK(is_engine_created() != true);
   engine_repp = new ECA_ENGINE (session_repp->connected_chainsetup_repp);
+  DBC_CHECK(is_engine_created() == true);
 
   /* to relay the batchmode parameter to created new thread */
   req_batchmode_rep = batchmode;
@@ -359,12 +351,14 @@ void ECA_CONTROL_BASE::start_engine_sub(bool batchmode)
     engine_repp = 0;
     delete engine_tmp;
   }
+
+  DBC_ENSURE(is_engine_created() == true);
 }
 
 /**
  * Routine used for launching the engine.
  */
-void ECA_CONTROL_BASE::run_engine(void)
+void ECA_CONTROL::run_engine(void)
 {
   last_exec_res_rep = 0;
   last_exec_res_rep = engine_repp->exec(req_batchmode_rep);
@@ -375,11 +369,12 @@ void ECA_CONTROL_BASE::run_engine(void)
  * Closes the processing engine.
  *
  * ensure:
- *  is_engine_started() != true
+ *  is_engine_created() != true
+ *  is_engine_running() != true
  */
-void ECA_CONTROL_BASE::close_engine(void)
+void ECA_CONTROL::close_engine(void)
 {
-  if (is_engine_started() != true) return;
+  if (is_engine_created() != true) return;
 
   engine_repp->command(ECA_ENGINE::ep_exit, 0.0);
 
@@ -421,7 +416,8 @@ void ECA_CONTROL_BASE::close_engine(void)
   }
 
   // ---
-  DBC_ENSURE(is_engine_started() != true);
+  DBC_ENSURE(is_engine_created() != true);
+  DBC_ENSURE(is_engine_running() != true);
   // ---
 }
 
@@ -430,7 +426,7 @@ void ECA_CONTROL_BASE::close_engine(void)
  *
  * @pre is_selected()
  */
-bool ECA_CONTROL_BASE::is_valid(void) const
+bool ECA_CONTROL::is_valid(void) const
 {
   // --------
   DBC_REQUIRE(is_selected());
@@ -444,7 +440,7 @@ bool ECA_CONTROL_BASE::is_valid(void) const
 /**
  * Returns true if active chainsetup exists and is connected.
  */
-bool ECA_CONTROL_BASE::is_connected(void) const
+bool ECA_CONTROL::is_connected(void) const
 {
   if (session_repp->connected_chainsetup_repp == 0) {
     return false;
@@ -457,25 +453,25 @@ bool ECA_CONTROL_BASE::is_connected(void) const
 /**
  * Returns true if some chainsetup is selected.
  */
-bool ECA_CONTROL_BASE::is_selected(void) const { return selected_chainsetup_repp != 0; } 
+bool ECA_CONTROL::is_selected(void) const { return selected_chainsetup_repp != 0; } 
 
 /**
  * Returns true if processing engine is running.
  */
-bool ECA_CONTROL_BASE::is_running(void) const { return (is_engine_started() == true && engine_repp->status() == ECA_ENGINE::engine_status_running); } 
+bool ECA_CONTROL::is_running(void) const { return (is_engine_created() == true && engine_repp->status() == ECA_ENGINE::engine_status_running); } 
 
 /**
  * Returns true if engine has finished processing. Engine state is 
  * either "finished" or "error".
  */
-bool ECA_CONTROL_BASE::is_finished(void) const
+bool ECA_CONTROL::is_finished(void) const
 {
-  return (is_engine_started() == true && 
+  return (is_engine_created() == true && 
 	  (engine_repp->status() == ECA_ENGINE::engine_status_finished ||
 	   engine_repp->status() == ECA_ENGINE::engine_status_error)); 
 } 
 
-string ECA_CONTROL_BASE::resource_value(const string& key) const
+string ECA_CONTROL::resource_value(const string& key) const
 { 
   ECA_RESOURCES ecarc;
   return ecarc.resource(key); 
@@ -486,7 +482,7 @@ string ECA_CONTROL_BASE::resource_value(const string& key) const
  *
  * @pre is_selected() == true
  */
-SAMPLE_SPECS::sample_pos_t ECA_CONTROL_BASE::length_in_samples(void) const
+SAMPLE_SPECS::sample_pos_t ECA_CONTROL::length_in_samples(void) const
 {
   // --------
   DBC_REQUIRE(is_selected());
@@ -508,7 +504,7 @@ SAMPLE_SPECS::sample_pos_t ECA_CONTROL_BASE::length_in_samples(void) const
  *
  * @pre is_selected() == true
  */
-double ECA_CONTROL_BASE::length_in_seconds_exact(void) const
+double ECA_CONTROL::length_in_seconds_exact(void) const
 {
   // --------
   DBC_REQUIRE(is_selected());
@@ -530,7 +526,7 @@ double ECA_CONTROL_BASE::length_in_seconds_exact(void) const
  *
  * @pre is_selected() == true
  */
-SAMPLE_SPECS::sample_pos_t ECA_CONTROL_BASE::position_in_samples(void) const
+SAMPLE_SPECS::sample_pos_t ECA_CONTROL::position_in_samples(void) const
 {
   // --------
   DBC_REQUIRE(is_selected());
@@ -544,7 +540,7 @@ SAMPLE_SPECS::sample_pos_t ECA_CONTROL_BASE::position_in_samples(void) const
  *
  * @pre is_selected() == true
  */
-double ECA_CONTROL_BASE::position_in_seconds_exact(void) const
+double ECA_CONTROL::position_in_seconds_exact(void) const
 {
   // --------
   DBC_REQUIRE(is_selected());
@@ -554,35 +550,52 @@ double ECA_CONTROL_BASE::position_in_seconds_exact(void) const
 }
 
 /**
- * Returns true if engine has been started. 
+ * Returns true if engine object has been created.
+ * If true, the engine object is available for use, but 
+ * the related engine thread is not necessarily running.
  *
- * Note: when in batch mode, status of notready means that
- *       engine has exited and is no longer receiving any commands
- *       and thus false is returned
+ * @see is_engine_running()
  */
-bool ECA_CONTROL_BASE::is_engine_started(void) const
+bool ECA_CONTROL::is_engine_created(void) const
 {
-  bool ret = true;
+  return (engine_repp != 0);
+}
 
-  if (engine_repp == 0) {
-    ret = false;
-  }
-  else {
-    if (engine_repp->batch_mode() == true &&
-	engine_repp->status() == ECA_ENGINE::engine_status_notready) {
-      ret = false;
-    }
-  }
-  
-  return ret;
+/**
+ * Returns true if engine is running and ready to receive
+ * control commands via the message queue.
+ * 
+ * In practise running means that the engine thread 
+ * has been created and it is running the exec() method
+ * of the engine.
+ * 
+ * @see is_engine_created()
+ */
+bool ECA_CONTROL::is_engine_running(void) const
+{
+  bool started = is_engine_created();
+
+  if (started != true)
+    return false;
+
+  /* note: has been started, but run_engine() has returned */
+  if (engine_pid_rep < 0)
+    return false;
+
+  DBC_CHECK(engine_repp != 0);
+  if (engine_repp->status() ==
+      ECA_ENGINE::engine_status_notready)
+    return false;
+
+  return true;
 }
 
 /**
  * Return info about engine status.
  */
-string ECA_CONTROL_BASE::engine_status(void) const
+string ECA_CONTROL::engine_status(void) const
 {
-  if (is_engine_started() == true) {
+  if (is_engine_created() == true) {
     switch(engine_repp->status()) {
     case ECA_ENGINE::engine_status_running: 
       {
@@ -613,144 +626,74 @@ string ECA_CONTROL_BASE::engine_status(void) const
   return "not started";
 }
 
-/**
- * Get a string containing a comma separated list of all chains 
- * attached to input with index 'aiod'. 
- *
- * @pre is_selected() == true
- */
-string ECA_CONTROL_BASE::attached_chains_input(AUDIO_IO* aiod) const
+void ECA_CONTROL::set_last_string(const list<string>& s)
 {
-  // --------
-  DBC_REQUIRE(is_selected() == true);
-  // --------
+  string s_rep;
 
-  vector<string> t = selected_chainsetup_repp->get_attached_chains_to_input(aiod);
-  string out = "";
-  vector<string>::const_iterator p = t.begin();
-  while(p != t.end()) {
-    out += *p;
-    ++p;
-    if (p != t.end()) out += ",";
-  }
-  return out;
-}
-
-/**
- * Get a string containing a comma separated list of all chains 
- * attached to output with index 'aiod'.
- *
- * @pre is_selected() == true
- */
-string ECA_CONTROL_BASE::attached_chains_output(AUDIO_IO* aiod) const
-{
-  // --------
-  DBC_REQUIRE(is_selected() == true);
-  // --------
-
-  vector<string> t = selected_chainsetup_repp->get_attached_chains_to_output(aiod);
-  string out = "";
-  vector<string>::const_iterator p = t.begin();
-  while(p != t.end()) {
-    out += *p;
-    ++p;
-    if (p != t.end()) out += ",";
-  }
-  return out;
-}
-
-/**
- * Get a string containing a comma separated list of all chains 
- * attached to audio object with name 'filename'. 
- *
- * @pre is_selected() == true
- */
-vector<string> ECA_CONTROL_BASE::attached_chains(const string& filename) const
-{
-  // --------
-  DBC_REQUIRE(is_selected() == true);
-  // --------
-
-  return selected_chainsetup_repp->get_attached_chains_to_iodev(filename);
-}
-
-void ECA_CONTROL_BASE::set_last_string(const list<string>& s)
-{
-  // use resize() instead of clear(); clear() was a late
-  // addition to C++ standard and not supported by all
-  // compilers (for example egcs-2.91.66) 
-  last_s_rep.resize(0);
-
-  DBC_CHECK(last_s_rep.size() == 0);
+  DBC_CHECK(s_rep.size() == 0);
   list<string>::const_iterator p = s.begin();
   while(p != s.end()) {
-    last_s_rep += *p;
+    s_rep += *p;
     ++p;
-    if (p != s.end()) last_s_rep += "\n";
+    if (p != s.end()) s_rep += "\n";
   }
-  last_type_rep = "s";
+  set_last_string(s_rep);
 }
 
-void ECA_CONTROL_BASE::set_last_string_list(const vector<string>& s)
+void ECA_CONTROL::set_last_string_list(const vector<string>& s)
 {
-  last_los_rep = s; 
-  last_type_rep = "S";
+  last_retval_rep.type = eci_return_value::retval_string_list;
+  last_retval_rep.string_list_val = s;
 }
 
-void ECA_CONTROL_BASE::set_last_string(const string& s)
+void ECA_CONTROL::set_last_string(const string& s)
 {
-  last_s_rep = s; 
-  last_type_rep = "s";
+  last_retval_rep.type = eci_return_value::retval_string;
+  last_retval_rep.string_val = s;
 }
 
-void ECA_CONTROL_BASE::set_last_float(double v)
+void ECA_CONTROL::set_last_float(double v)
 {
-  last_f_rep = v; 
-  last_type_rep = "f";
+  last_retval_rep.type = eci_return_value::retval_float;
+  last_retval_rep.m.float_val = v;
 }
 
-void ECA_CONTROL_BASE::set_last_integer(int v)
+void ECA_CONTROL::set_last_integer(int v)
 {
-  last_i_rep = v; 
-  last_type_rep = "i";
+  last_retval_rep.type = eci_return_value::retval_integer;
+  last_retval_rep.m.int_val = v;
 }
  
-void ECA_CONTROL_BASE::set_last_long_integer(long int v)
+void ECA_CONTROL::set_last_long_integer(long int v)
 {
-  last_li_rep = v; 
-  last_type_rep = "li";
+  last_retval_rep.type = eci_return_value::retval_long_integer;
+  last_retval_rep.m.long_int_val = v;
 }
 
-void ECA_CONTROL_BASE::set_last_error(const string& s)
+void ECA_CONTROL::set_last_error(const string& s)
 {
-  last_error_rep = s;
-  last_type_rep = "e";
+  last_retval_rep.type = eci_return_value::retval_error;
+  last_retval_rep.string_val = s;
 }
 
-const vector<string>& ECA_CONTROL_BASE::last_string_list(void) const { return last_los_rep; }
-const string& ECA_CONTROL_BASE::last_string(void) const { return last_s_rep; }
-double ECA_CONTROL_BASE::last_float(void) const { return last_f_rep; }
-int ECA_CONTROL_BASE::last_integer(void) const { return last_i_rep; } 
-long int ECA_CONTROL_BASE::last_long_integer(void) const { return last_li_rep; }
-const string& ECA_CONTROL_BASE::last_error(void) const { return last_error_rep; }
-const string& ECA_CONTROL_BASE::last_type(void) const { return last_type_rep; }
+string ECA_CONTROL::last_error(void) const
+{
+  if (last_retval_rep.type == eci_return_value::retval_error)
+    return last_retval_rep.string_val;
 
-void ECA_CONTROL_BASE::clear_last_values(void)
+  return string();
+}
+
+void ECA_CONTROL::clear_last_values(void)
 { 
-  last_los_rep.clear();
-  last_s_rep = "";
-  last_li_rep = 0;
-  last_i_rep = 0;
-  last_f_rep = 0.0f;
-  last_error_rep = "";
-  last_type_rep = "-";
+  ECA_CONTROL_MAIN::clear_return_value(&last_retval_rep);
 }
 
-void ECA_CONTROL_BASE::set_float_to_string_precision(int precision)
+void ECA_CONTROL::set_float_to_string_precision(int precision)
 {
   float_to_string_precision_rep = precision;
 }
-std::string ECA_CONTROL_BASE::float_to_string(double n) const
+std::string ECA_CONTROL::float_to_string(double n) const
 {
   return kvu_numtostr(n, float_to_string_precision_rep);
 }
