@@ -1,7 +1,8 @@
 // ------------------------------------------------------------------------
 // eca-control.cpp: Class for controlling the whole ecasound library
-// Copyright (C) 1999-2005,2008 Kai Vehmanen
+// Copyright (C) 1999-2005,2008,2009 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
+// Copyright (C) 2009 Adam Linson
 //
 // Attributes:
 //     eca-style-version: 3 (see Ecasound Programmer's Guide)
@@ -17,14 +18,14 @@
 // GNU General Public License for more details.
 // 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ------------------------------------------------------------------------
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <cassert>
 #include <iostream>
 #include <fstream>
 #include <string>
@@ -43,9 +44,13 @@
 #include <kvu_dbc.h>
 #include <kvu_numtostr.h>
 
-#include "eca-control.h"
+#include "audioio.h"
+#include "eca-chain.h"
 #include "eca-chainop.h"
 #include "eca-chainsetup.h"
+#include "eca-control.h"
+#include "eca-control-main.h"
+#include "eca-engine.h"
 #include "eca-object-factory.h"
 #include "eca-object-map.h"
 #include "eca-preset-map.h"
@@ -71,6 +76,7 @@ using std::list;
 using std::vector;
 using std::cerr;
 using std::endl;
+using namespace ECA;
 
 /**
  * Declarations for private static functions
@@ -82,19 +88,40 @@ static string eca_aio_register_sub(ECA_OBJECT_MAP& objmap);
  */
 
 ECA_CONTROL::ECA_CONTROL (ECA_SESSION* psession) 
-  : ECA_CONTROL_OBJECTS(psession),
-    wellformed_mode_rep(false),
-    ctrl_dump_rep(this)
+  : ctrl_dump_rep(this),
+    wellformed_mode_rep(false)
 {
   ECA_LOG_MSG(ECA_LOGGER::system_objects, "ECA_CONTROL constructor");
+
+  session_repp = psession;
+  selected_chainsetup_repp = psession->selected_chainsetup_repp;
+  engine_repp = 0;
+  engine_pid_rep = -1;
+  engine_exited_rep.set(0);
+  float_to_string_precision_rep = 3;
+  joining_rep = false;
+  DBC_CHECK(is_engine_created() != true);
+
+  selected_audio_object_repp = 0;
+  selected_audio_input_repp = 0;
+  selected_audio_output_repp = 0;
 }
 
 ECA_CONTROL::~ECA_CONTROL(void)
 {
   ECA_LOG_MSG(ECA_LOGGER::system_objects, "ECA_CONTROL destructor");
+  close_engine();
 }
 
-void ECA_CONTROL::command(const string& cmd_and_args)
+void ECA_CONTROL::fill_command_retval(struct eci_return_value *retval) const
+{
+  if (retval == 0)
+    return;
+
+  *retval = last_retval_rep;
+}
+
+void ECA_CONTROL::command(const string& cmd_and_args, struct eci_return_value *retval)
 {
   clear_last_values();
   clear_action_arguments();
@@ -140,6 +167,8 @@ void ECA_CONTROL::command(const string& cmd_and_args)
       }
     }
   }
+
+  fill_command_retval(retval);
 }
 
 void ECA_CONTROL::set_action_argument(const string& s)
@@ -224,13 +253,14 @@ SAMPLE_SPECS::sample_pos_t ECA_CONTROL::first_action_argument_as_samples(void) c
 #endif
 }
 
-void ECA_CONTROL::command_float_arg(const string& cmd, double arg)
+void ECA_CONTROL::command_float_arg(const string& cmd, double arg, struct eci_return_value *retval)
 {
   clear_action_arguments();
   set_action_argument(arg);
   int action_id = ec_unknown;
   action_id = ECA_IAMODE_PARSER::command_to_action_id(cmd);
   action(action_id);
+  fill_command_retval(retval);
 }
 
 /**
@@ -317,8 +347,11 @@ void ECA_CONTROL::check_action_preconditions(int action_id)
     }
     else {
       if (is_valid() == true) {
-	ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: No chainsetup connected. Trying to connect currently selected chainsetup.");
-	connect_chainsetup();
+	ECA_LOG_MSG(ECA_LOGGER::info, 
+		    "NOTE: No chainsetup connected. Trying to connect currently selected chainsetup \""
+		    + selected_chainsetup_repp->name() 
+		    + "\"");
+	connect_chainsetup(0);
       }
       if (is_connected() != true) {
 	/* connect_chainsetup() sets last_error() so we just add to it */
@@ -347,6 +380,37 @@ void ECA_CONTROL::action(int action_id,
   action(action_id);
 }
 
+bool ECA_CONTROL::action_helper_check_cop_op_args(int copid, int coppid)
+{
+  DBC_REQUIRE(is_selected() == true);
+
+  const vector<string>& selchains =
+    selected_chainsetup_repp->selected_chains();
+  bool res = false;
+
+  if (selchains.size() == 0) {
+    set_last_error("No chain selected, unable to identify chainop");
+  }
+  else if (selchains.size() > 1) {
+    set_last_error("More than one chain selected, unable to identify chainop");
+  }
+  else {
+    const CHAIN* selch = 
+      selected_chainsetup_repp->get_chain_with_name(selchains[0]);
+    if (copid < 1 || copid > selch->number_of_chain_operators()) {
+      set_last_error("Invalid chainop-id, unable to identify chainop");
+    }
+    else if (coppid < 1) {
+      set_last_error("Invalid copp-id, indexing starts from 1.");
+    }
+    else {
+      res = true;
+    }
+  }
+
+  return res;
+}
+
 void ECA_CONTROL::action(int action_id)
 {
   clear_last_values();
@@ -427,7 +491,7 @@ void ECA_CONTROL::action(int action_id)
   case ec_cs_connect: 
     { 
       if (is_valid() != false) {
-	connect_chainsetup(); 
+	connect_chainsetup(0); 
       }
       else {
 	set_last_error("Can't connect; chainsetup not valid!");
@@ -628,15 +692,37 @@ void ECA_CONTROL::action(int action_id)
       int id2 = atoi(a[1].c_str());
       CHAIN_OPERATOR::parameter_t v = atof(a[2].c_str());
 
-      if (id1 > 0 && id2 > 0) {
+      bool valid = 
+	action_helper_check_cop_op_args(id1, id2);
+      if (valid == true) {
 	select_chain_operator(id1);
 	select_chain_operator_parameter(id2);
 	set_chain_operator_parameter(v);
       }
-      else
-	set_last_error("Chain operator indexing starts from 1.");
+      // note: helper func sets the error string if needed
       break; 
     }
+  case ec_cop_get:
+    {
+      vector<string> a =
+	kvu_string_to_vector(first_action_argument_as_string(), ',');
+      if (a.size() < 2) {
+        set_last_error("Not enough parameters!");
+        break;
+      }
+      int id1 = atoi(a[0].c_str());
+      int id2 = atoi(a[1].c_str());
+      bool valid = 
+	action_helper_check_cop_op_args(id1, id2);
+      if (valid == true) {
+        select_chain_operator(id1);
+        select_chain_operator_parameter(id2);
+        set_last_float(get_chain_operator_parameter());
+      }
+      // note: helper func sets the error string if needed
+      break;
+    }
+
   case ec_cop_status: 
     { 
       set_last_string(chain_operator_status()); 
@@ -696,8 +782,20 @@ void ECA_CONTROL::action(int action_id)
   // ---
   // Engine commands
   // ---
-  case ec_engine_launch: { if (is_engine_started() != true) engine_start(); break; }
-  case ec_engine_halt: { close_engine(); break; }
+  case ec_engine_launch: { 
+    if (is_engine_running() != true) 
+      engine_start(); 
+    else
+      set_last_error("Engine already running, use 'engine-halt' first.");
+    break; 
+  }
+  case ec_engine_halt: {
+    if (is_engine_running() == true)
+      close_engine(); 
+    else
+      set_last_error("Engine not running, use 'engine-launch' first.");
+    break; 
+  }
   case ec_engine_status: { set_last_string(engine_status()); break; }
 
   // ---
@@ -782,7 +880,7 @@ void ECA_CONTROL::action(int action_id)
       set_last_error("Can't reconnect chainsetup.");
     }
     else {
-      connect_chainsetup();
+      connect_chainsetup(0);
       if (selected_chainsetup() != connected_chainsetup()) {
 	set_last_error("Can't reconnect chainsetup.");
       }
@@ -796,48 +894,90 @@ void ECA_CONTROL::action(int action_id)
   }
 }
 
-string ECA_CONTROL::last_value_to_string(void)
+/**
+ * Executes chainsetup edit on connect chainsetup.
+ * 
+ * @pre is_connected()
+ */
+bool ECA_CONTROL::execute_edit_on_connected(const chainsetup_edit_t& edit)
 {
-  string type = last_type();
-  string result;
+  DBC_REQUIRE(is_connected() == true);
 
-  if (type == "e") {
-    result = last_error();
+  bool retval = false;
+  
+  if (is_engine_running() == true) {
+    ECA_ENGINE::complex_command_t engine_cmd;
+    engine_cmd.type = ECA_ENGINE::ep_exec_edit;
+    engine_cmd.m.cs = edit;
+    engine_repp->command(engine_cmd);
+    retval = true;
   }
-  else if (type == "s")
-    result = last_string();
-  else if (type == "S") {
-    result = kvu_vector_to_string(kvu_vector_search_and_replace(last_string_list(), ",", "\\,"), ",");
+  else {
+    /* note: engine not yet running, execute edit directly */
+    retval = session_repp->connected_chainsetup_repp->execute_edit(edit);
   }
-  else if (type == "i")
-    result = kvu_numtostr(last_integer());
-  else if (type == "li")
-    result = kvu_numtostr(last_long_integer());
-  else if (type == "f")
-    result = ECA_CONTROL_BASE::float_to_string(last_float());
 
-  return result;
+  return retval;
 }
 
-void ECA_CONTROL::print_last_value(void)
+/**
+ * Executes chainsetup edit on selected chainsetup.
+ * 
+ * @param edit object specifying the edit action
+ * @param index if non-negative, override the chainsetup selection
+ */
+bool ECA_CONTROL::execute_edit_on_selected(const chainsetup_edit_t& edit, int index)
 {
-  string type = last_type();
-  string result;
+  bool retval = false;
 
-  if (type == "e") {
+  ECA_CHAINSETUP *csetup = 0;
+
+  if (index < 0) {
+    csetup = selected_chainsetup_repp;
+  }
+  else {
+    if (index >= 0 &&
+	index < static_cast<int>(session_repp->chainsetups_rep.size())) {
+      csetup = session_repp->chainsetups_rep[index];
+    }
+  }
+
+  /* note: make sure that if the selected chainsetup is 
+   *       in use by the engine, the edit is performed 
+   *       by the engine thread! 
+   */
+  if (csetup != 0) {
+    if (csetup->is_enabled() == true &&
+	is_engine_running() == true) {
+      execute_edit_on_connected(edit);
+    }
+    else {
+      csetup->execute_edit(edit);
+    }
+  }
+
+  return retval;
+}
+
+void ECA_CONTROL::print_last_value(struct eci_return_value *retval) const
+{
+  std::string result;
+
+  if (retval->type == eci_return_value::retval_error) {
     result += "ERROR: ";
   }
 
-  result += last_value_to_string();
+  result += ECA_CONTROL_MAIN::return_value_to_string(retval);
 
   if (wellformed_mode_rep != true) {
-    if (result.size() > 0) {
+    if (result.size() > 0) 
       ECA_LOG_MSG(ECA_LOGGER::eiam_return_values, result);
-    }
   }
   else {
     /* in wellformed-output-mode we always create return output */
-    ECA_LOG_MSG(ECA_LOGGER::eiam_return_values, type + " " + result);
+    ECA_LOG_MSG(ECA_LOGGER::eiam_return_values, 
+		std::string(return_value_type_to_string(retval)) +
+		" " + result);
   }
 }
 
@@ -860,7 +1000,7 @@ string ECA_CONTROL::chainsetup_details_to_string(const ECA_CHAINSETUP* cs) const
     result += "connected to engine (engine status: ";
     result += engine_status() + ")";
   }
-  else if (cs->is_enabled() && is_engine_started() == true) {
+  else if (cs->is_enabled() && is_engine_created() == true) {
     result += "connected (engine status: ";
     result += engine_status() + ")";
   }
@@ -891,7 +1031,6 @@ string ECA_CONTROL::chainsetup_details_to_string(const ECA_CHAINSETUP* cs) const
       result += ECA_OBJECT_FACTORY::audio_object_to_eos(cs->inputs[idx], "i");
     result += " ";
     result += (*chain_citer)->to_string();
-    result += " ";
     idx = (*chain_citer)->connected_output();
     if (idx >= 0)
       result += ECA_OBJECT_FACTORY::audio_object_to_eos(cs->outputs[idx], "o");
@@ -985,7 +1124,7 @@ string ECA_CONTROL::chain_operator_status(void) const
 	msg << "[" << n + 1 << "] ";
 	msg << cop->get_parameter_name(n + 1);
 	msg << " ";
-	msg << ECA_CONTROL_BASE::float_to_string(cop->get_parameter(n + 1));
+	msg << float_to_string(cop->get_parameter(n + 1));
 	if (n + 1 < cop->number_of_params()) msg <<  ", ";
       }
       st_info_string = cop->status();
@@ -1023,7 +1162,7 @@ string ECA_CONTROL::controller_status(void) const
 	mitem << "\n\t\t[" << n + 1 << "] ";
 	mitem << gtrl->get_parameter_name(n + 1);
 	mitem << " ";
-	mitem << ECA_CONTROL_BASE::float_to_string(gtrl->get_parameter(n + 1));
+	mitem << float_to_string(gtrl->get_parameter(n + 1));
 	if (n + 1 < gtrl->number_of_params()) mitem <<  ", ";
       }
       st_info_string = gtrl->status();
@@ -1304,15 +1443,15 @@ void ECA_CONTROL::operator_descriptions_helper(const ECA_OBJECT_MAP& arg, string
 	/* 5.2 description */
 	*result += "," + kvu_string_search_and_replace(pd.description, ',', '_');
 	/* 5.3 default value */
-	*result += "," + ECA_CONTROL_BASE::float_to_string(pd.default_value);
+	*result += "," + float_to_string(pd.default_value);
 	/* 5.4 is bounded above (1=yes, 0=no) */
 	*result += ",above=" + kvu_numtostr(static_cast<int>(pd.bounded_above));
 	/* 5.5 upper bound */
-	*result += ",upper=" + ECA_CONTROL_BASE::float_to_string(pd.upper_bound);
+	*result += ",upper=" + float_to_string(pd.upper_bound);
 	/* 5.6 is bounded below (1=yes, 0=no) */
 	*result += ",below=" + kvu_numtostr(static_cast<int>(pd.bounded_below));
 	/* 5.7 lower bound */
-	*result += ",lower=" + ECA_CONTROL_BASE::float_to_string(pd.lower_bound);
+	*result += ",lower=" + float_to_string(pd.lower_bound);
 	/* 5.8. is toggled (1=yes, 0=no) */
 	*result += "," + kvu_numtostr(static_cast<int>(pd.toggled));
 	/* 5.9. is integer value (1=yes, 0=no) */
