@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-engine.cpp: Main processing engine
-// Copyright (C) 1999-2008 Kai Vehmanen
+// Copyright (C) 1999-2009 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
@@ -53,6 +53,7 @@
 #include "eca-chainop.h"
 #include "eca-error.h"
 #include "eca-logger.h"
+#include "eca-chainsetup-edit.h"
 #include "eca-engine.h"
 #include "eca-engine_impl.h"
 
@@ -111,11 +112,16 @@ int ECA_ENGINE_DEFAULT_DRIVER::exec(ECA_ENGINE* engine, ECA_CHAINSETUP* csetup)
       engine_repp->engine_iteration();
     }
     else {
-      /* case 3a: engine finished and in batch mode -> exit */
-      if ((engine_repp->status() == ECA_ENGINE::engine_status_finished ||
-	   engine_repp->status() == ECA_ENGINE::engine_status_error) &&
+      /* case 3a-i: engine finished and in batch mode -> exit */
+      if (engine_repp->status() == ECA_ENGINE::engine_status_finished &&
 	  engine_repp->batch_mode() == true) {
-	/* batch operation finished (or error occured) */
+	ECA_LOG_MSG(ECA_LOGGER::system_objects, "batch finished in exec, exit");
+	break;
+      }
+
+      /* case 3a-ii: engine error occured -> exit */
+      else if (engine_repp->status() == ECA_ENGINE::engine_status_error) {
+	ECA_LOG_MSG(ECA_LOGGER::system_objects, "engine error, exit");
 	break;
       }
 
@@ -274,7 +280,7 @@ int ECA_ENGINE::exec(bool batch_mode)
 
   batchmode_enabled_rep = batch_mode;
 
-  ECA_LOG_MSG(ECA_LOGGER::subsystems, "Engine init - Driver start");
+  ECA_LOG_MSG(ECA_LOGGER::subsystems, "Engine - Driver start");
 
   int res = driver_repp->exec(this, csetup_repp);
   if (res < 0) {
@@ -325,7 +331,25 @@ int ECA_ENGINE::exec(bool batch_mode)
  */
 void ECA_ENGINE::command(Engine_command_t cmd, double arg)
 {
-  impl_repp->command_queue_rep.push_back(static_cast<int>(cmd),arg);
+  ECA_ENGINE::complex_command_t item;
+  item.type = cmd;
+  item.m.legacy.value = arg;
+  impl_repp->command_queue_rep.push_back(item);
+}
+
+/**
+ * Sends 'ccmd' to engines command queue. Commands are 
+ * processed in the server's main loop. Passing a complex
+ * command allows to address objects regardless of
+ * the state of ECA_CHAINSETUP iterators (i.e. currently
+ * selected objects).
+ *
+ * context: C-level-0
+ *          must no be called from exec() context
+ */
+void ECA_ENGINE::command(complex_command_t ccmd)
+{
+  impl_repp->command_queue_rep.push_back(ccmd);
 }
 
 /**
@@ -458,7 +482,72 @@ ECA_ENGINE::Engine_status_t ECA_ENGINE::status(void) const
  */
 void ECA_ENGINE::check_command_queue(void)
 {
-  interpret_queue();
+  while(impl_repp->command_queue_rep.is_empty() != true) {
+    ECA_ENGINE::complex_command_t item;
+    int popres = impl_repp->command_queue_rep.pop_front(&item);
+
+    if (popres <= 0) {
+      /* queue is empty or temporarily unavailable, unable to continue 
+       * processing messages without blocking */
+      break;
+    }
+
+    switch(item.type) 
+      {
+	// ---
+	// Basic commands.
+	// ---            
+      case ep_exit:
+	{
+	  edit_lock_rep = true;
+	  if (status() == engine_status_running || 
+	      status() == engine_status_finished) request_stop();
+	  impl_repp->command_queue_rep.clear();
+	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"ecasound_queue: exit!");
+	  driver_repp->exit();
+	  return;
+	}
+
+	// ---
+	// Chain operators (stateless addressing)
+	// ---
+      case ep_exec_edit: {
+	csetup_repp->execute_edit(item.m.cs);
+	break;
+      }
+
+      case ep_prepare: { if (is_prepared() != true) prepare_operation(); break; }
+      case ep_start: { if (status() != engine_status_running) request_start(); break; }
+      case ep_stop: { if (status() == engine_status_running || 
+			  status() == engine_status_finished) request_stop(); break; }
+	
+	// ---
+	// Edit locks
+	// ---            
+      case ep_edit_lock: { edit_lock_rep = true; break; }
+      case ep_edit_unlock: { edit_lock_rep = false; break; }
+	
+	// ---
+	// Section/chain (en/dis)abling commands.
+	// ---
+      case ep_c_select: { csetup_repp->selected_chain_index_rep = static_cast<size_t>(item.m.legacy.value); break; }
+      case ep_c_muting: { chain_muting(); break; }
+      case ep_c_bypass: { chain_processing(); break; }
+
+	// ---
+	// Global position
+	// ---
+      case ep_rewind: { change_position(- item.m.legacy.value); break; }
+      case ep_forward: { change_position(item.m.legacy.value); break; }
+      case ep_setpos: { set_position(item.m.legacy.value); break; }
+      case ep_setpos_samples: { set_position_samples(static_cast<SAMPLE_SPECS::sample_pos_t>(item.m.legacy.value)); break; }
+      case ep_setpos_live_samples: { set_position_samples_live(static_cast<SAMPLE_SPECS::sample_pos_t>(item.m.legacy.value)); break; }
+
+      case ep_debug: break;
+
+      } /* switch */
+    
+  }
 }
 
 /**
@@ -489,6 +578,10 @@ void ECA_ENGINE::init_engine_state(void)
   finished_rep = false;
   inputs_not_finished_rep = 1; // for the 1st iteration
   outputs_finished_rep = 0;
+  mixslot_repp->event_tag_set(SAMPLE_BUFFER::tag_end_of_stream, false);
+  for(size_t n = 0; n < cslots_rep.size(); n++) {
+    cslots_rep[n]->event_tag_set(SAMPLE_BUFFER::tag_end_of_stream, false);
+  }
 }
 
 /**
@@ -519,7 +612,8 @@ void ECA_ENGINE::update_engine_state(void)
       // we are not allowed to call request_stop here
       command(ECA_ENGINE::ep_stop, 0.0f);
     }
-    finished_rep = true;
+
+    state_change_to_finished();
   }
   
   // --
@@ -787,7 +881,7 @@ int ECA_ENGINE::max_channels(void) const
  * Requests the engine driver to start operation.
  *
  * This function should only be called from 
- * interpret_queue().
+ * check_command_queue().
  * 
  * @pre status() != engine_status_running
  *
@@ -810,7 +904,7 @@ void ECA_ENGINE::request_start(void)
  * Requests the engine driver to stop operation.
  *
  * This function should only be called from 
- * interpret_queue().
+ * check_command_queue().
  *
  * @pre status() == ECA_ENGINE::engine_status_running ||
  *      status() == ECA_ENGINE::engine_status_finished
@@ -956,6 +1050,15 @@ void ECA_ENGINE::stop_forked_objects(void)
   priv_toggle_forked_objects(false, outputs_repp);
 }
 
+void ECA_ENGINE::state_change_to_finished(void)
+{
+  if (finished_rep != true) {
+    ECA_LOG_MSG_NOPREFIX(ECA_LOGGER::info, "");
+    ECA_LOG_MSG(ECA_LOGGER::subsystems, "Engine - Processing finished");
+  }
+  finished_rep = true;
+}
+
 void ECA_ENGINE::prepare_realtime_objects(void)
 {
   /* 1. prepare objects */
@@ -1144,132 +1247,11 @@ void ECA_ENGINE::posthandle_control_position(void)
 	  status() == ECA_ENGINE::engine_status_finished) {
 	command(ECA_ENGINE::ep_stop, 0.0f);
       }
-      finished_rep = true;
+      state_change_to_finished();
     }
   }
 }
 
-/**********************************************************************
- * Engine implementation - Private functions for cmd queue handling
- **********************************************************************/
-
-/**
- * Processes messages from the command queue.
- * If no messages are available, function
- * will return immediately.
- *
- * context: E-level-2
- *          can be run at the same time as engine_iteration(); 
- *          note! this is called with the engine lock held
- *          so no long operations allowed!
- *
- * @see check_command_queue()
- */
-void ECA_ENGINE::interpret_queue(void)
-{
-  while(impl_repp->command_queue_rep.is_empty() != true) {
-    const std::pair<int,double>* item_p = impl_repp->command_queue_rep.front();
-    std::pair<int,double> item;
-    if (item_p != impl_repp->command_queue_rep.invalid_item()) {
-      item = *item_p;
-    }
-    else {
-      /* queue temporarily unavailable, unable to continue processing
-       * messages */
-      break;
-    }
-
-    switch(item.first) 
-      {
-	// ---
-	// Basic commands.
-	// ---            
-      case ep_exit:
-	{
-	  edit_lock_rep = true;
-	  if (status() == engine_status_running || 
-	      status() == engine_status_finished) request_stop();
-	  while(impl_repp->command_queue_rep.is_empty() == false) impl_repp->command_queue_rep.pop_front();
-	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"ecasound_queue: exit!");
-	  driver_repp->exit();
-	  return;
-	}
-
-      case ep_prepare: { if (is_prepared() != true) prepare_operation(); break; }
-      case ep_start: { if (status() != engine_status_running) request_start(); break; }
-      case ep_stop: { if (status() == engine_status_running || 
-			  status() == engine_status_finished) request_stop(); break; }
-	
-	// ---
-	// Edit locks
-	// ---            
-      case ep_edit_lock: { edit_lock_rep = true; break; }
-      case ep_edit_unlock: { edit_lock_rep = false; break; }
-	
-	// ---
-	// Section/chain (en/dis)abling commands.
-	// ---
-      case ep_c_select: { csetup_repp->active_chain_index_rep = static_cast<size_t>(item.second); break; }
-      case ep_c_muting: { chain_muting(); break; }
-      case ep_c_bypass: { chain_processing(); break; }
-	
-	// ---
-	// Chain operators
-	// ---
-      case ep_cop_select: { 
-	csetup_repp->active_chainop_index_rep =  static_cast<size_t>(item.second);
-	if (csetup_repp->active_chainop_index_rep - 1 < static_cast<int>((*chains_repp)[csetup_repp->active_chain_index_rep]->number_of_chain_operators()))
-	  (*chains_repp)[csetup_repp->active_chain_index_rep]->select_chain_operator(csetup_repp->active_chainop_index_rep);
-	else 
-	  csetup_repp->active_chainop_index_rep = 0;
-	break;
-      }
-      case ep_copp_select: { 
-	csetup_repp->active_chainop_param_index_rep = static_cast<size_t>(item.second);
-	(*chains_repp)[csetup_repp->active_chain_index_rep]->select_chain_operator_parameter(csetup_repp->active_chainop_param_index_rep);
-	break;
-      }
-      case ep_copp_value: { 
-	assert(chains_repp != 0);
-	(*chains_repp)[csetup_repp->active_chain_index_rep]->set_parameter(item.second);
-	break;
-      }
-	
-	// ---
-	// Controllers
-	// ---
-      case ep_ctrl_select: { 
-	csetup_repp->active_ctrl_index_rep =  static_cast<size_t>(item.second);
-	if (csetup_repp->active_ctrl_index_rep - 1 < static_cast<int>((*chains_repp)[csetup_repp->active_chain_index_rep]->number_of_controllers()))
-	  (*chains_repp)[csetup_repp->active_chain_index_rep]->select_controller(csetup_repp->active_ctrl_index_rep);
-	else 
-	  csetup_repp->active_ctrl_index_rep = 0;
-	break;
-      }
-      case ep_ctrlp_select: { 
-	csetup_repp->active_ctrl_param_index_rep = static_cast<size_t>(item.second);
-	(*chains_repp)[csetup_repp->active_chain_index_rep]->select_controller_parameter(csetup_repp->active_ctrl_param_index_rep);
-	break;
-      }
-      case ep_ctrlp_value: { 
-	assert(chains_repp != 0);
-	(*chains_repp)[csetup_repp->active_chain_index_rep]->set_controller_parameter(item.second);
-	break;
-      }
-	// ---
-	// Global position
-	// ---
-      case ep_rewind: { change_position(- item.second); break; }
-      case ep_forward: { change_position(item.second); break; }
-      case ep_setpos: { set_position(item.second); break; }
-      case ep_setpos_samples: { set_position_samples(static_cast<SAMPLE_SPECS::sample_pos_t>(item.second)); break; }
-      case ep_setpos_live_samples: { set_position_samples_live(static_cast<SAMPLE_SPECS::sample_pos_t>(item.second)); break; }
-      } /* switch */
-    
-    impl_repp->command_queue_rep.pop_front();
-  }
-}
-
 /**********************************************************************
  * Engine implementation - Private functions for setup and cleanup
  **********************************************************************/
@@ -1362,8 +1344,8 @@ void ECA_ENGINE::init_servers(void)
  */
 void ECA_ENGINE::init_chains(void)
 {
-
   mixslot_repp->number_of_channels(max_channels());
+  mixslot_repp->event_tag_set(SAMPLE_BUFFER::tag_mixed_content);
 
   cslots_rep.resize(chains_repp->size());
   for(size_t n = 0; n < cslots_rep.size(); n++) {
@@ -1638,8 +1620,7 @@ void ECA_ENGINE::inputs_to_chains(void)
 	else {
 	  /* case-1b: input connected to chain 'n', copy 'mixslot' to 
 	   *          the matching per-chain slot */
-	  cslots_rep[c]->number_of_channels(mixslot_repp->number_of_channels());
-	  cslots_rep[c]->copy(*mixslot_repp);
+	  cslots_rep[c]->copy_all_content(*mixslot_repp);
 	}
       }
     }
@@ -1665,7 +1646,7 @@ void mix_to_outputs_divide_helper(const SAMPLE_BUFFER *from, SAMPLE_BUFFER *to,
     if (from->number_of_channels() < to->number_of_channels()) {
       to->make_silent();
     }
-    to->copy(*from);
+    to->copy_matching_channels(*from);
     to->divide_by(divide_by);
   }
   else {
@@ -1680,10 +1661,10 @@ void mix_to_outputs_sum_helper(const SAMPLE_BUFFER *from, SAMPLE_BUFFER *to, boo
     if (from->number_of_channels() < to->number_of_channels()) {
       to->make_silent();
     }
-    to->copy(*from);
+    to->copy_matching_channels(*from);
   }
   else {
-    to->add(*from);
+    to->add_matching_channels(*from);
   }
 }
 
@@ -1744,6 +1725,8 @@ void ECA_ENGINE::mix_to_outputs(bool skip_realtime_target_outputs)
  	  else
 	    mix_to_outputs_sum_helper(cslots_rep[n], mixslot_repp, (count == 1));
 
+	  mixslot_repp->event_tags_add(*cslots_rep[n]);
+
 	  if (count == output_chain_count_rep[outputnum]) {
 	    (*outputs_repp)[outputnum]->write_buffer(mixslot_repp);
 	    if ((*outputs_repp)[outputnum]->finished() == true) 
@@ -1768,10 +1751,10 @@ void ECA_ENGINE::mix_to_outputs(bool skip_realtime_target_outputs)
  */
 void ECA_ENGINE::chain_muting(void)
 {
-  if ((*chains_repp)[csetup_repp->active_chain_index_rep]->is_muted()) 
-    (*chains_repp)[csetup_repp->active_chain_index_rep]->toggle_muting(false);
+  if ((*chains_repp)[csetup_repp->selected_chain_index_rep]->is_muted()) 
+    (*chains_repp)[csetup_repp->selected_chain_index_rep]->toggle_muting(false);
   else
-    (*chains_repp)[csetup_repp->active_chain_index_rep]->toggle_muting(true);
+    (*chains_repp)[csetup_repp->selected_chain_index_rep]->toggle_muting(true);
 }
 
 /**
@@ -1779,10 +1762,10 @@ void ECA_ENGINE::chain_muting(void)
  */
 void ECA_ENGINE::chain_processing(void)
 {
-  if ((*chains_repp)[csetup_repp->active_chain_index_rep]->is_processing()) 
-    (*chains_repp)[csetup_repp->active_chain_index_rep]->toggle_processing(false);
+  if ((*chains_repp)[csetup_repp->selected_chain_index_rep]->is_processing()) 
+    (*chains_repp)[csetup_repp->selected_chain_index_rep]->toggle_processing(false);
   else
-    (*chains_repp)[csetup_repp->active_chain_index_rep]->toggle_processing(true);
+    (*chains_repp)[csetup_repp->selected_chain_index_rep]->toggle_processing(true);
 }
 
 /**********************************************************************
