@@ -1,9 +1,12 @@
 // ------------------------------------------------------------------------
 // eca-iamode-parser.cpp: Class that handles registering and querying 
 //                        interactive mode commands.
-// Copyright (C) 1999-2005 Kai Vehmanen
+// Copyright (C) 1999-2005,2008 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
+// Attributes:
+//     eca-style-version: 3 (see Ecasound Programmer's Guide)
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
@@ -19,6 +22,10 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 // ------------------------------------------------------------------------
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <map>
 #include <vector>
 #include <string>
@@ -64,9 +71,10 @@ const map<string,int>& ECA_IAMODE_PARSER::registered_commands(void)
       register_commands_ctrl();
       register_commands_ctrlp();
       register_commands_dump();
+      register_commands_external();
     }
   }
-  return(*cmd_map_repp);
+  return *cmd_map_repp;
 }
 
 vector<string> ECA_IAMODE_PARSER::registered_commands_list(void)
@@ -78,7 +86,7 @@ vector<string> ECA_IAMODE_PARSER::registered_commands_list(void)
     cmdlist.push_back(p->first);
     ++p;
   }
-  return(cmdlist);
+  return cmdlist;
 }
 
 void ECA_IAMODE_PARSER::register_commands_misc(void)
@@ -267,6 +275,7 @@ void ECA_IAMODE_PARSER::register_commands_cop(void)
   (*cmd_map_repp)["cop-register"] = ec_cop_register;
   (*cmd_map_repp)["cop-selected"] = ec_cop_selected;
   (*cmd_map_repp)["cop-set"] = ec_cop_set;
+  (*cmd_map_repp)["cop-get"] = ec_cop_get;
   (*cmd_map_repp)["cop-status"] = ec_cop_status;
 }
 
@@ -324,9 +333,18 @@ void ECA_IAMODE_PARSER::register_commands_dump(void)
   (*cmd_map_repp)["dump-cop-value"] = ec_dump_cop_value;
 }
 
+void ECA_IAMODE_PARSER::register_commands_external(void)
+{
+#if ECA_COMPILE_JACK
+  (*cmd_map_repp)["jack-connect"] = ec_jack_connect;
+  (*cmd_map_repp)["jack-disconnect"] = ec_jack_disconnect;
+  (*cmd_map_repp)["jack-list-connections"] = ec_jack_list_connections;
+#endif
+}
+
 int ECA_IAMODE_PARSER::command_to_action_id(const std::string& cmdstring)
 {
-  return((*cmd_map_repp)[cmdstring]);
+  return (*cmd_map_repp)[cmdstring];
 }
 
 bool ECA_IAMODE_PARSER::action_requires_params(int id)
@@ -374,6 +392,7 @@ bool ECA_IAMODE_PARSER::action_requires_params(int id)
   case ec_cop_add:
   case ec_cop_select:
   case ec_cop_set:
+  case ec_cop_get:
 
   case ec_copp_select:
   case ec_copp_set:
@@ -386,12 +405,15 @@ bool ECA_IAMODE_PARSER::action_requires_params(int id)
   case ec_dump_target:
   case ec_dump_cop_value:
 
-    return(true);
+  case ec_jack_connect:
+  case ec_jack_disconnect:
+
+    return true;
     
   default: 
     break;
   }
-  return(false);
+  return false;
 }
 
 bool ECA_IAMODE_PARSER::action_requires_connected(int id)
@@ -405,12 +427,12 @@ bool ECA_IAMODE_PARSER::action_requires_connected(int id)
   case ec_cs_disconnect:
   case ec_cs_set_position:
   case ec_cs_set_position_samples:
-    return(true);
+    return true;
     
   default: 
     break;
   }
-  return(false);
+  return false;
 }
 
 bool ECA_IAMODE_PARSER::action_requires_selected(int id)
@@ -490,6 +512,7 @@ bool ECA_IAMODE_PARSER::action_requires_selected(int id)
   case ec_cop_select:
   case ec_cop_selected:
   case ec_cop_set:
+  case ec_cop_get:
   case ec_cop_status:
 
   case ec_copp_list:
@@ -502,12 +525,12 @@ bool ECA_IAMODE_PARSER::action_requires_selected(int id)
   case ec_ctrl_selected:
   case ec_ctrl_status:
 
-    return(true);
+    return true;
 
   default: break;
   }
   
-  return(false);
+  return false;
 }
 
 bool ECA_IAMODE_PARSER::action_requires_selected_not_connected(int id)
@@ -544,12 +567,12 @@ bool ECA_IAMODE_PARSER::action_requires_selected_not_connected(int id)
   case ec_ao_set_position_samples:
   case ec_ao_wave_edit:
 
-    return(true);
+    return true;
     
   default: 
     break;
   }
-  return(false);
+  return false;
 
 }
 
@@ -570,12 +593,12 @@ bool ECA_IAMODE_PARSER::action_requires_selected_audio_input(int id)
   case ec_ai_get_length_samples:
   case ec_ai_get_format:
   case ec_ai_wave_edit:
-    return(true);
+    return true;
     
   default: 
     break;
   }
-  return(false);
+  return false;
 
 }
 
@@ -596,12 +619,12 @@ bool ECA_IAMODE_PARSER::action_requires_selected_audio_output(int id)
   case ec_ao_get_length_samples:
   case ec_ao_get_format:
   case ec_ao_wave_edit:
-    return(true);
+    return true;
     
   default: 
     break;
   }
-  return(false);
+  return false;
 
 }
 
