@@ -605,10 +605,15 @@ GENERIC_CONTROLLER* ECA_OBJECT_FACTORY::create_controller (const string& argu)
 std::string ECA_OBJECT_FACTORY::probe_default_output_device(void)
 {
     ECA_RESOURCES ecaresources;
-    string default_output = ecaresources.resource("default-output");
+    const char *output_autodetect = "autodetect";
+    string default_output = output_autodetect;
     bool output_selected = true;
 
-    if (default_output == "autodetect") {
+    if (ecaresources.has("default-output") == true)
+      default_output =
+	ecaresources.resource("default-output");
+
+    if (default_output == output_autodetect) {
       
       output_selected = false;
 
@@ -761,7 +766,7 @@ string ECA_OBJECT_FACTORY::operator_parameters_to_eos(const OPERATOR* chainop)
 string ECA_OBJECT_FACTORY::audio_object_to_eos(const AUDIO_IO* aiod, const std::string& direction)
 {
   MESSAGE_ITEM t;
-  t << " -" << direction << ":";
+  t << "-" << direction << ":";
   for(int n = 0; n < aiod->number_of_params(); n++) {
     /* step: if parameter has commas, or whitespace, quote the whole parameter */
     std::string param = aiod->get_parameter(n + 1);
