@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audioio_jack_manager.cpp: Manager for JACK client objects
-// Copyright (C) 2001-2004,2008 Kai Vehmanen
+// Copyright (C) 2001-2004,2008,2009 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3
@@ -1049,13 +1049,16 @@ int AUDIO_IO_JACK_MANAGER::exec(ECA_ENGINE* engine, ECA_CHAINSETUP* csetup)
       break;
     }
 
-    /* case 2: engine finished and in batch mode -> exit */
-    if ((engine_repp->status() == ECA_ENGINE::engine_status_finished ||
-	 engine_repp->status() == ECA_ENGINE::engine_status_error) &&
-	engine->batch_mode() == true) {
-
-      /* batch operation finished (or error occured) */
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "batch finished in exec");
+    /* case 2-i: engine finished and in batch mode -> exit */
+    if (engine_repp->status() == ECA_ENGINE::engine_status_finished &&
+	engine_repp->batch_mode() == true) {
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "batch finished in exec, exit");
+      break;
+    }
+    
+    /* case 2-ii: engine error occured -> exit */
+    else if (engine_repp->status() == ECA_ENGINE::engine_status_error) {
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "engine error, exit");
       break;
     }
 
@@ -1067,15 +1070,24 @@ int AUDIO_IO_JACK_MANAGER::exec(ECA_ENGINE* engine, ECA_CHAINSETUP* csetup)
     }
   }
 
+  DEBUG_CFLOW_STATEMENT(cerr << "jack_exec: out of exec" << endl);
+
   if (is_connection_active() == true) {
     deactivate_server_connection();
   }
 
+  DEBUG_CFLOW_STATEMENT(cerr << "jack_exec: deactivated" << endl);
+
+  pthread_mutex_lock(&engine_mod_lock_rep);
+  exit_request_rep = 0;
   engine_repp = 0;
+  pthread_mutex_unlock(&engine_mod_lock_rep);
 
   /* signal exit() that we are done */
   signal_exit();
 
+  DEBUG_CFLOW_STATEMENT(cerr << "jack_exec: exit" << endl);
+
   return result;
 }
 
@@ -1620,7 +1632,7 @@ long int AUDIO_IO_JACK_MANAGER::read_samples(int client_id, void* target_buffer,
 void AUDIO_IO_JACK_MANAGER::write_samples(int client_id, void* target_buffer, long int samples)
 {
   // DEBUG_CFLOW_STATEMENT(cerr << endl << "write_samples:" << client_id);
-
+  size_t sample_size = sizeof(jack_default_audio_sample_t);
   long int writesamples = (samples <= buffersize_rep) ? samples : buffersize_rep;
   jack_default_audio_sample_t* ptr =
     static_cast<jack_default_audio_sample_t*>(target_buffer);
@@ -1629,8 +1641,12 @@ void AUDIO_IO_JACK_MANAGER::write_samples(int client_id, void* target_buffer, lo
   list<eca_jack_port_data*>::const_iterator p = node->ports.begin();
   while(p != node->ports.end()) {
     if ((*p)->cb_buffer != 0) {
-      memcpy((*p)->cb_buffer, ptr, writesamples * sizeof(jack_default_audio_sample_t));
+      /* note: cb_buffer points to jack_default_audio_sample_t* */
+      memcpy((*p)->cb_buffer, ptr, writesamples * sample_size);
       ptr += writesamples;
+      memset((*p)->cb_buffer + writesamples,
+	     0,
+	     (buffersize_rep - writesamples) * sample_size);
     }
     ++p;
   }
