@@ -1,12 +1,16 @@
 // ------------------------------------------------------------------------
 // samplebuffer.cpp: Class representing a buffer of audio samples.
-// Copyright (C) 1999-2005 Kai Vehmanen
+// Copyright (C) 1999-2005,2009 Kai Vehmanen
 //
 // Attributes:
 //     eca-style-version: 3
 //
 // References:
+//   - libsamplerate:
 //     http://www.mega-nerd.com/SRC/
+//   - liboil:
+//     http://liboil.freedesktop.org/
+//     http://library.gnome.org/devel/liboil/
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,9 +34,9 @@
 #include <iostream>
 #include <vector>
 
-#include <cmath>   /* ceil(), floor() */
-#include <cstring> /* memcpy */
-#include <cstdlib> /* labs() */
+#include <cmath>    /* ceil(), floor() */
+#include <cstring>  /* memcpy */
+#include <stdlib.h> /* not cstdlib we need e.g. posix_memalign() */
 
 #include <sys/types.h>
 
@@ -43,6 +47,12 @@
 #include <samplerate.h>
 #endif
 
+#ifdef ECA_USE_LIBOIL 
+extern "C" {
+#include <liboil/liboil.h>
+}
+#endif
+
 #include "eca-sample-conversion.h"
 #include "samplebuffer.h"
 #include "samplebuffer_impl.h"
@@ -63,6 +73,19 @@ static const bool is_system_littleendian = false;
 static const bool is_system_littleendian = true;
 #endif
 
+using namespace std;
+
+static void priv_alloc_sample_buf(SAMPLE_SPECS::sample_t **memptr, size_t size)
+{
+#ifdef HAVE_POSIX_MEMALIGN
+  /* align buffers to 128bit/16octet boundary */
+  posix_memalign(reinterpret_cast<void**>(memptr), 16, size);
+#else
+  *memptr = reinterpret_cast<SAMPLE_SPECS::sample_t*>(malloc(size));
+#endif
+
+}
+
 /**
  * Constructs a new sample buffer object.
  */
@@ -76,11 +99,17 @@ SAMPLE_BUFFER::SAMPLE_BUFFER (buf_size_t buffersize, channel_size_t channels)
   DBC_REQUIRE(channels >= 0);
   // ---
 
+  /* catch if someone changes SAMPLE_SPECS::silent_value */
+  DBC_DECLARE(float f = 0.0f);
+  DBC_CHECK(static_cast<uint32_t>(f) == 0);
+  DBC_CHECK(sizeof(f) == 4);
+
   impl_repp = new SAMPLE_BUFFER_impl;
 
   buffer.resize(channels);
   for(size_t n = 0; n < buffer.size(); n++) {
-    buffer[n] = new sample_t [reserved_samples_rep];
+    priv_alloc_sample_buf(&buffer[n], 
+			  sizeof(sample_t) * reserved_samples_rep);
   }
   make_silent();
 
@@ -93,6 +122,10 @@ SAMPLE_BUFFER::SAMPLE_BUFFER (buf_size_t buffersize, channel_size_t channels)
 #else
   impl_repp->quality_rep = 5;
 #endif
+
+#ifdef ECA_USE_LIBOIL
+  oil_init();
+#endif
  
   ECA_LOG_MSG(ECA_LOGGER::functions, 
 		"Buffer created, channels: " +
@@ -104,89 +137,6 @@ SAMPLE_BUFFER::SAMPLE_BUFFER (buf_size_t buffersize, channel_size_t channels)
   // ---
 }
 
-/**
- * Constructs a new sample buffer object from a reference
- * to an already existing object.
- *
- * For better performance, doesn't copy IO-buffers nor
- * iterator state.
- *
- * ** Note! This function is obsolete!
- */
-SAMPLE_BUFFER::SAMPLE_BUFFER (const SAMPLE_BUFFER& x)
-  : channel_count_rep(x.channel_count_rep),
-    buffersize_rep(x.buffersize_rep),
-    reserved_samples_rep(x.reserved_samples_rep)
-{
-  impl_repp = new SAMPLE_BUFFER_impl;
-
-  buffer.resize(x.buffer.size());
-
-  for(size_t n = 0; n < buffer.size(); n++) {
-    buffer[n] = new sample_t [reserved_samples_rep];
-    std::memcpy(buffer[n], x.buffer[n], buffersize_rep * sizeof(sample_t));
-  }
-
-  impl_repp->old_buffer_repp = 0;
-
-  ECA_LOG_MSG(ECA_LOGGER::functions, 
-		"Buffer copy-constructed, channels: " +
-		kvu_numtostr(buffer.size()) + ", length-samples: " +
-		kvu_numtostr(buffersize_rep) + ".");
-
-  // ---
-  DBC_ENSURE(buffer.size() == static_cast<size_t>(channel_count_rep));
-  DBC_ENSURE(buffersize_rep == reserved_samples_rep);
-  // ---
-}
-
-/**
- * Assignment operator.
- *
- * For better performance, doesn't copy IO-buffers nor
- * iterator state.
- *
- * ** Note! This function is obsolete!
- */
-SAMPLE_BUFFER& SAMPLE_BUFFER::operator=(const SAMPLE_BUFFER& x)
-{
-  if (this != &x) {
-
-    impl_repp->resample_memory_rep = x.impl_repp->resample_memory_rep;
-    
-    if (x.buffersize_rep > reserved_samples_rep ||
-	x.buffer.size() != buffer.size()) {
-
-      reserved_samples_rep = x.buffersize_rep;
-
-      for(size_t n = 0; n < buffer.size(); n++) delete[] buffer[n];
-
-      buffer.resize(x.buffer.size());
-
-      for(size_t n = 0; n < buffer.size(); n++) {
-	buffer[n] = new sample_t [reserved_samples_rep];
-	for(buf_size_t m = 0; m < reserved_samples_rep; m++) {
-	  buffer[n][m] = SAMPLE_SPECS::silent_value;
-	}
-      }
-    }
-    
-    buffersize_rep = x.buffersize_rep;
-    channel_count_rep = x.channel_count_rep;
-
-    for(size_t n = 0; n < buffer.size(); n++) {
-      std::memcpy(buffer[n], x.buffer[n], buffersize_rep * sizeof(sample_t));
-    }
-  }
-
-  return *this;
-
-  // ---
-  DBC_ENSURE(buffer.size() == static_cast<size_t>(channel_count_rep));
-  DBC_ENSURE(buffersize_rep == reserved_samples_rep);
-  // ---
-}
-
 /**
  * Destructor.
  */
@@ -196,13 +146,13 @@ SAMPLE_BUFFER::~SAMPLE_BUFFER (void)
 
   for(size_t n = 0; n < buffer.size(); n++) {
     if (buffer[n] != 0) {
-      delete[] buffer[n];
+      ::free(buffer[n]);
       buffer[n] = 0;
     }
   }
 
   if (impl_repp->old_buffer_repp != 0) {
-    delete[] impl_repp->old_buffer_repp;
+    ::free(impl_repp->old_buffer_repp);
     impl_repp->old_buffer_repp = 0;
   }
 
@@ -220,10 +170,35 @@ SAMPLE_BUFFER::~SAMPLE_BUFFER (void)
 
 /**
  * Channel-wise addition. Buffer length is increased if necessary.
+ * Only channels, that are present in both source and destination, are 
+ * modified.
  *
+ * Note: event tags are not copied!
+ * 
  * @post length_in_samples() >= x.length_in_samples()
  */
-void SAMPLE_BUFFER::add(const SAMPLE_BUFFER& x)
+void SAMPLE_BUFFER::add_matching_channels(const SAMPLE_BUFFER& x)
+{
+#ifdef ECA_USE_LIBOIL 
+  if (x.length_in_samples() > length_in_samples()) {
+    length_in_samples(x.length_in_samples());
+  }
+  int min_c_count = (channel_count_rep <= x.channel_count_rep) ? channel_count_rep : x.channel_count_rep;
+  for(channel_size_t q = 0; q < min_c_count; q++) {
+    oil_add_f32(reinterpret_cast<float*>(buffer[q]),
+		reinterpret_cast<const float*>(buffer[q]),
+		reinterpret_cast<const float*>(x.buffer[q]),
+		x.length_in_samples());
+  }
+#else
+  add_matching_channels_ref(x);
+#endif
+}
+
+/**
+ * Unoptimized version of add_matching_channels().
+ */
+void SAMPLE_BUFFER::add_matching_channels_ref(const SAMPLE_BUFFER& x)
 {
   if (x.length_in_samples() > length_in_samples()) {
     length_in_samples(x.length_in_samples());
@@ -239,7 +214,11 @@ void SAMPLE_BUFFER::add(const SAMPLE_BUFFER& x)
 /**
  * Channel-wise, weighted addition. Before addition every sample is 
  * multiplied by '1/weight'. Buffer length is increased if necessary.
+ * Only channels, that are present in both source and destination, are 
+ * modified.
  *
+ * Note: event tags are not copied!
+ * 
  * @pre weight != 0
  * @post length_in_samples() >= x.length_in_samples()
  */
@@ -249,6 +228,9 @@ void SAMPLE_BUFFER::add_with_weight(const SAMPLE_BUFFER& x, int weight)
   DBC_REQUIRE(weight != 0);
   // ---
 
+  /* note: gcc does a suprisingly good job for this function,
+   *       so additional optimizations don't seem worthwhile */
+
   if (x.length_in_samples() > length_in_samples()) {
     length_in_samples(x.length_in_samples());
   }
@@ -263,68 +245,148 @@ void SAMPLE_BUFFER::add_with_weight(const SAMPLE_BUFFER& x, int weight)
 /**
  * Channel-wise copy. Buffer length is adjusted if necessary.
  *
+ * Note: event tags are not copied!
+ * 
  * @post length_in_samples() == x.length_in_samples()
  */
-void SAMPLE_BUFFER::copy(const SAMPLE_BUFFER& x)
+void SAMPLE_BUFFER::copy_matching_channels(const SAMPLE_BUFFER& x)
 {
   length_in_samples(x.length_in_samples());
   
   int min_c_count = (channel_count_rep <= x.channel_count_rep) ? channel_count_rep : x.channel_count_rep;
   for(channel_size_t q = 0; q < min_c_count; q++) {
-    for(buf_size_t t = 0; t < length_in_samples(); t++) {
-      buffer[q][t] = x.buffer[q][t];
-    }
+    std::memcpy(buffer[q], 
+		x.buffer[q],
+		sizeof(sample_t) * length_in_samples());
+
+    /* ref implementation:
+     * for(buf_size_t t = 0; t < length_in_samples(); t++) {
+     *   buffer[q][t] = x.buffer[q][t];
+     * }
+     */ 
   }
 }
 
 /**
- * Ranged channel-wise copy. Copies samples in range 
- * 'start_pos' - 'end_pos' from buffer 'x' to current 
- * buffer position 'to_pos'. 
+ * Copy all audio contents from 'x'. After copying, length, channel
+ * count, and event tag set match to those of 'x'.
  *
+ * @post length_in_samples() == x.length_in_samples()
+ * @post number_of_channels() == number_of_channels()
+ * @post for all I: event_tag_test(I) == x.event_tag_test(I)
+ */
+void SAMPLE_BUFFER::copy_all_content(const SAMPLE_BUFFER& x)
+{
+  length_in_samples(x.length_in_samples());
+  number_of_channels(x.number_of_channels());
+  
+  for(channel_size_t q = 0; q < number_of_channels(); q++) {
+    std::memcpy(buffer[q], 
+		x.buffer[q],
+		sizeof(sample_t) * length_in_samples());
+
+    /* ref implementation:
+     * {
+     * for(buf_size_t t = 0; t < length_in_samples(); t++) {
+     * 	buffer[q][t] = x.buffer[q][t];
+     * }
+     */
+  }
+  
+  event_tags_set(x);
+}
+
+/**
+ * Ranged channel-wise copy. Copies samples in range 
+ * 'start_pos' - 'end_pos-1' from buffer 'x' to current 
+ * buffer position 'to_pos'. The 'src' object must have
+ * equal number of channels as the current object.
+ *
+ * Note: event tags are not copied!
+ * 
  * @pre start_pos <= end_pos
+ * @pre 
  * @pre to_pos < length_in_samples()
  */
-void SAMPLE_BUFFER::copy_range(const SAMPLE_BUFFER& x, 
-			       buf_size_t start_pos,
-			       buf_size_t end_pos,
-			       buf_size_t to_pos) 
+void SAMPLE_BUFFER::copy_range(const SAMPLE_BUFFER& src, 
+			       buf_size_t src_start_pos,
+			       buf_size_t src_end_pos,
+			       buf_size_t dst_to_pos) 
 {
   // ---
-  DBC_REQUIRE(start_pos <= end_pos);
-  DBC_REQUIRE(to_pos < length_in_samples());
+  DBC_REQUIRE(src_start_pos <= src_end_pos);
+  DBC_REQUIRE(dst_to_pos < length_in_samples());
+  DBC_REQUIRE(number_of_channels() == src.number_of_channels());
   // ---
 
-  int min_c_count = (channel_count_rep <= x.channel_count_rep) ? channel_count_rep : x.channel_count_rep;
+  if (src_end_pos > src.length_in_samples())
+    src_end_pos = src.length_in_samples();
 
-  if (start_pos >= x.length_in_samples()) start_pos = x.length_in_samples();
-  if (end_pos >= x.length_in_samples()) end_pos = x.length_in_samples();
+  for(channel_size_t q = 0; q < channel_count_rep; q++) {
+    buf_size_t dst_i = dst_to_pos;
+    for(buf_size_t src_i = src_start_pos; 
+	  src_i < src_end_pos && 
+	  dst_i < length_in_samples();
+	src_i++, dst_i++) {
 
-  buf_size_t to_index = to_pos;
+      buffer[q][dst_i] = src.buffer[q][src_i];
 
-  for(channel_size_t q = 0; q < min_c_count; q++) {
-    for(buf_size_t s = start_pos; 
-	  s < end_pos && 
-	  s < x.length_in_samples() &&
-	  to_index < length_in_samples() &&
-	  to_index < x.length_in_samples();
-	s++) {
-      buffer[q][to_index] = x.buffer[q][s];
-      ++to_index;
+#ifdef SAMPLEBUFFER_COPY_RANGE_VERBOSE_DEBUG
+      if (dst_i == 0 || src_i == 0 || src_i == src_end_pos - 1 ||dst_i == length_in_samples() - 1) {
+	std::fprintf(stderr, "dst[%d][%ld] = src[%d][%ld]\n", 
+		     q, dst_i, q, src_i);
+      }
+#endif
     }
   }
 }
 
+void SAMPLE_BUFFER::multiply_by(SAMPLE_BUFFER::sample_t factor, int channel)
+{
+#ifdef ECA_USE_LIBOIL 
+  oil_scalarmultiply_f32_ns(reinterpret_cast<float*>(buffer[channel]), 
+			    reinterpret_cast<const float*>(buffer[channel]), 
+			    reinterpret_cast<const float*>(&factor), 
+			    buffersize_rep);
+#else
+  multiply_by_ref(factor, channel);
+#endif
+}
+
+void SAMPLE_BUFFER::multiply_by(SAMPLE_BUFFER::sample_t factor)
+{
+  for(channel_size_t n = 0; n < channel_count_rep; n++) {
+    multiply_by(factor, n);
+  }
+}
+
+void SAMPLE_BUFFER::multiply_by_ref(SAMPLE_BUFFER::sample_t factor)
+{
+  for(channel_size_t n = 0; n < channel_count_rep; n++) {
+    for(buf_size_t m = 0; m < buffersize_rep; m++) {
+      buffer[n][m] *= factor;
+    }
+  }
+}
+
+void SAMPLE_BUFFER::multiply_by_ref(SAMPLE_BUFFER::sample_t factor, int channel)
+{
+  for(buf_size_t m = 0; m < buffersize_rep; m++) {
+    buffer[channel][m] *= factor;
+  }
+}
+
 /**
  * Divides all samples by 'dvalue'.
  */
 void SAMPLE_BUFFER::divide_by(SAMPLE_BUFFER::sample_t dvalue)
 {
-  for(channel_size_t n = 0; n < channel_count_rep; n++) {
-    for(buf_size_t m = 0; m < buffersize_rep; m++) {
-      buffer[n][m] /= dvalue;
-    }
-  }
+  multiply_by(1.0 / dvalue);
+}
+
+void SAMPLE_BUFFER::divide_by_ref(SAMPLE_BUFFER::sample_t dvalue)
+{
+  multiply_by_ref(1.0 / dvalue);
 }
 
 /**
@@ -336,15 +398,44 @@ void SAMPLE_BUFFER::make_empty(void)
   SAMPLE_BUFFER::length_in_samples(0);
 }
 
+/**
+ * Mutes one channel  of the buffer.
+ *
+ * @pre channel >= 0 
+ * @pre channel < number_of_channels()
+ */
+void SAMPLE_BUFFER::make_silent(int channel)
+{
+  DBC_REQUIRE(channel >= 0);
+  DBC_REQUIRE(channel < number_of_channels());
+
+  std::memset(buffer[channel], 0, buffersize_rep * sizeof(SAMPLE_SPECS::silent_value));
+
+  /* - the generic version: */
+  /*
+    for(buf_size_t s = 0; s < buffersize_rep; s++) {
+    buffer[n][s] = SAMPLE_SPECS::silent_value;
+    }
+  */
+}
+
+/**
+ * Unoptimized version of make_silent(int).
+ */
+void SAMPLE_BUFFER::make_silent_ref(int channel)
+{
+  for(buf_size_t s = 0; s < buffersize_rep; s++) {
+    buffer[channel][s] = SAMPLE_SPECS::silent_value;
+  }
+}
+
 /**
  * Mutes the whole buffer.
  */
 void SAMPLE_BUFFER::make_silent(void)
 {
   for(channel_size_t n = 0; n < channel_count_rep; n++) {
-    for(buf_size_t s = 0; s < buffersize_rep; s++) {
-      buffer[n][s] = SAMPLE_SPECS::silent_value;
-    }
+    make_silent(n);
   }
 }
 
@@ -361,6 +452,24 @@ void SAMPLE_BUFFER::make_silent_range(buf_size_t start_pos,
   DBC_REQUIRE(end_pos >= 0);
   // --
 
+  for(channel_size_t n = 0; n < channel_count_rep; n++) {
+    std::memset(buffer[n] + start_pos, 
+		0,
+		sizeof(sample_t) * 
+		(end_pos < buffersize_rep ? end_pos : buffersize_rep));
+  }
+}
+
+/**
+ * Unoptimized version of make_silent_range()
+ */
+void SAMPLE_BUFFER::make_silent_range_ref(buf_size_t start_pos,
+					  buf_size_t end_pos) {
+  // --
+  DBC_REQUIRE(start_pos >= 0);
+  DBC_REQUIRE(end_pos >= 0);
+  // --
+
   for(channel_size_t n = 0; n < channel_count_rep; n++) {
     for(buf_size_t s = start_pos; s < end_pos && s < buffersize_rep; s++) {
       buffer[n][s] = SAMPLE_SPECS::silent_value;
@@ -381,6 +490,27 @@ void SAMPLE_BUFFER::limit_values(void)
 	buffer[n][m] = SAMPLE_SPECS::impl_min_value;
     }
   }
+  
+#if 0 /* slower than the naive implementation */
+  {
+    float min = SAMPLE_SPECS::impl_min_value;
+    float max = SAMPLE_SPECS::impl_max_value;
+    for(channel_size_t n = 0; n < channel_count_rep; n++) {
+      oil_clip_f32(reinterpret_cast<float*>(buffer[n]),
+		   sizeof(float),
+		   reinterpret_cast<const float*>(buffer[n]),
+		   sizeof(float),
+		   buffersize_rep,
+		   &min, &max);
+    }
+  }
+#endif
+}
+
+/** Unoptimized version of limit_values() */
+void SAMPLE_BUFFER::limit_values_ref(void)
+{
+  limit_values();
 }
 
 /**
@@ -865,7 +995,7 @@ void SAMPLE_BUFFER::number_of_channels(channel_size_t len)
     size_t old_size = buffer.size();
     buffer.resize(len);
     for(channel_size_t n = old_size; n < len; n++) {
-      buffer[n] = new sample_t [reserved_samples_rep];
+      priv_alloc_sample_buf(&buffer[n], sizeof(sample_t) * reserved_samples_rep);
     }
     ECA_LOG_MSG(ECA_LOGGER::functions, "Increasing channel-count (1).");    
   }
@@ -889,11 +1019,14 @@ void SAMPLE_BUFFER::number_of_channels(channel_size_t len)
 /**
  * Sets the length of buffer in samples.
  *
- * Note! If length is increased, the added samples
+ * Note: if length is increased, the added samples
  *       are muted.
  */
 void SAMPLE_BUFFER::length_in_samples(buf_size_t len)
 {
+  DBC_REQUIRE(len >= 0);
+  DBC_CHECK(buffersize_rep <= reserved_samples_rep);
+
   if (len > reserved_samples_rep) {
 
     DBC_CHECK(impl_repp->rt_lock_rep != true);
@@ -901,19 +1034,23 @@ void SAMPLE_BUFFER::length_in_samples(buf_size_t len)
 
     reserved_samples_rep = len * 2;
     for(size_t n = 0; n < buffer.size(); n++) {
-      delete[] buffer[n];
-      buffer[n] = new sample_t [reserved_samples_rep];
+      sample_t *prev_buffer = buffer[n];
+      priv_alloc_sample_buf(&buffer[n], sizeof(sample_t) * reserved_samples_rep);
+      for (buf_size_t m = 0; m < buffersize_rep; m++)
+	buffer[n][m] = prev_buffer[m];
+      ::free(prev_buffer);
     }
 
     if (impl_repp->old_buffer_repp != 0) {
-      delete[] impl_repp->old_buffer_repp;
-      impl_repp->old_buffer_repp = new sample_t [reserved_samples_rep];
+      ::free(impl_repp->old_buffer_repp);
+      priv_alloc_sample_buf(&impl_repp->old_buffer_repp, sizeof(sample_t) * reserved_samples_rep);
     }
   }
 
   if (len > buffersize_rep) {
     for(size_t n = 0; n < buffer.size(); n++) {
-      for(buf_size_t m = buffersize_rep; m < len; m++) {
+      /* note: mute starting from 'buffersize_rep' */
+      for(buf_size_t m = buffersize_rep; m < reserved_samples_rep; m++) {
 	buffer[n][m] = SAMPLE_SPECS::silent_value;
       }
     }
@@ -956,8 +1093,8 @@ void SAMPLE_BUFFER::resample_init_memory(SAMPLE_SPECS::sample_rate_t from_srate,
 #endif
 
     for(int c = 0; c < channel_count_rep; c++) {
-      delete[] buffer[c];
-      buffer[c] = new sample_t [reserved_samples_rep];
+      ::free(buffer[c]);
+      priv_alloc_sample_buf(&buffer[c], sizeof(sample_t) * reserved_samples_rep);
     }
   }
 
@@ -976,7 +1113,7 @@ void SAMPLE_BUFFER::resample_init_memory(SAMPLE_SPECS::sample_rate_t from_srate,
 #ifdef ECA_DEBUG_MODE
     DBC_CHECK(impl_repp->rt_lock_rep != true);
 #endif
-    impl_repp->old_buffer_repp = new sample_t [reserved_samples_rep];
+    priv_alloc_sample_buf(&impl_repp->old_buffer_repp, sizeof(sample_t) * reserved_samples_rep);
   }
 
   if (impl_repp->resample_memory_rep.size() < static_cast<size_t>(channel_count_rep)) {
@@ -1043,6 +1180,48 @@ void SAMPLE_BUFFER::release_pointer_reflock(void)
   DBC_ENSURE(impl_repp->lockref_rep >= 0);
 }
 
+/**
+ * Adds all event tags that are set for 'sbuf' (bitwise-OR).
+ */
+void SAMPLE_BUFFER::event_tags_add(const SAMPLE_BUFFER& sbuf)
+{
+  impl_repp->event_tags_rep |= 
+    sbuf.impl_repp->event_tags_rep;
+}
+
+/**
+ * Sets only those event flags that are set for 'sbuf'.
+ */
+void SAMPLE_BUFFER::event_tags_set(const SAMPLE_BUFFER& sbuf)
+{
+  impl_repp->event_tags_rep = 
+    sbuf.impl_repp->event_tags_rep;
+}
+
+/**
+ * Clears all tags matching 'tagmask'
+ */
+void SAMPLE_BUFFER::event_tags_clear(Tag_name tagmask)
+{
+  event_tag_set(tagmask, false);
+}
+
+/** 
+ * Set/clears the event tag 'tag'.
+ */
+void SAMPLE_BUFFER::event_tag_set(Tag_name tag, bool val)
+{
+  if (val)
+    impl_repp->event_tags_rep |= tag;
+  else
+    impl_repp->event_tags_rep &= ~tag;
+}
+
+bool SAMPLE_BUFFER::event_tag_test(Tag_name tag)
+{
+  return (impl_repp->event_tags_rep & tag) ? true : false;
+}
+
 /**
  * Resamples samplebuffer contents.
  *
@@ -1210,7 +1389,7 @@ void SAMPLE_BUFFER::resample_secret_rabbit_code(SAMPLE_SPECS::sample_rate_t from
       params.output_frames_gen = 0;
       params.input_frames_used = old_buffer_size;
     }
-    DBC_CHECK(std::labs(params.input_frames_used - old_buffer_size) == 0);
+    DBC_CHECK(::labs(params.input_frames_used - old_buffer_size) == 0);
 #ifdef ECA_DEBUG_MODE
     /* make sure all input samples have been used */
     if (old_buffer_size != params.input_frames_used) { std::cerr << "input_frames_over=" << old_buffer_size - params.input_frames_used << ".\n"; }
