@@ -5,15 +5,12 @@
 
 /* FIXME: add check for big sync-error -> ecasound probably 
  *        died so better to give an error */
-/* FIXME: add some mechanism to detect failed eci_init(); */
 /* FIXME: add check for msgsize errors */
-/* FIXME: get rid of the all static string lengths (C... blaah ;)) */
-/* FIXME: add proper signal handling */
 
 /** ------------------------------------------------------------------------
  * ecasoundc.cpp: Standalone C implementation of the 
  *                ecasound control interface
- * Copyright (C) 2000-2006 Kai Vehmanen
+ * Copyright (C) 2000-2006,2008,2009 Kai Vehmanen
  * Copyright (C) 2003 Michael Ewe
  * Copyright (C) 2001 Aymeric Jeanneau
  *
@@ -34,6 +31,9 @@
  * -------------------------------------------------------------------------
  * History of major changes:
  *
+ * 2009-02-08 Kai Vehmanen
+ *     - Finally got rid of the fixed-size parsing buffers.
+ *     - Added handling (or proper ignoring) of SIGPIPE signals.
  * 2006-12-06 Kai Vehmanen 
  *     - Fixed severe string termination bug in handling lists of
  *       strings.
@@ -52,10 +52,12 @@
  * -------------------------------------------------------------------------
  */
 
+#include <assert.h>
 #include <stdio.h>        /* ANSI-C: printf(), ... */
 #include <stdlib.h>       /* ANSI-C: calloc(), free() */
 #include <string.h>       /* ANSI-C: strlen() */
 #include <errno.h>        /* ANSI-C: errno */
+#include <stdbool.h>
 
 #include <fcntl.h>        /* POSIX: fcntl() */
 #include <sys/poll.h>     /* XPG4-UNIX: poll() */
@@ -63,7 +65,7 @@
 #include <sys/stat.h>     /* POSIX: stat() */
 #include <sys/types.h>    /* POSIX: fork() */
 #include <sys/wait.h>     /* POSIX: wait() */
-#include <signal.h>       /* POSIIX: signal handling */
+#include <signal.h>       /* POSIX: signal handling */
 
 #include "ecasoundc.h"
 
@@ -71,16 +73,18 @@
  * Options
  */
 
-/* #define ECI_ENABLE_DEBUG */
+// #define ECI_ENABLE_DEBUG
 
 /* --------------------------------------------------------------------- 
  * Definitions and constants
  */
 
-#define ECI_MAX_PARSER_BUF_SIZE    524288 /* XXX: wasting memory, should be fixed */
+#define ECI_PARSER_BUF_SIZE        65536
+
+#define ECI_MAX_DYN_ALLOC_SIZE     16777216 /* assert if reached */
 #define ECI_MAX_FLOAT_BUF_SIZE     32
 #define ECI_MAX_RETURN_TYPE_SIZE   4
-#define ECI_MAX_STRING_SIZE        65536  /* XXX: see above */
+#define ECI_INI_STRING_SIZE        64
 #define ECI_MAX_RESYNC_ATTEMPTS    9
 #define ECI_MAX_LAST_COMMAND_SIZE  64
 
@@ -133,10 +137,17 @@
  * Data structures 
  */
 
+struct eci_string_s {
+  char *d;   /* buffer to string contents */
+  int slen;     /* string length in octets, including terminating null */
+  int size;     /* buffer length in octets, including terminating null */
+};
+typedef struct eci_string_s eci_string;
+
 struct eci_los_list {
   struct eci_los_list* prev_repp;
   struct eci_los_list* next_repp;
-  char* data_repp;
+  eci_string data_repp;
 };
 
 struct eci_parser { 
@@ -148,11 +159,12 @@ struct eci_parser {
   long int last_li_rep;
   int last_i_rep;
   int last_counter_rep;
-  char last_error_repp[ECI_MAX_STRING_SIZE];
   char last_type_repp[ECI_MAX_RETURN_TYPE_SIZE];
   struct eci_los_list* last_los_repp;
-  /* char* last_los_repp[ECI_MAX_STRING_SIZE]; */
-  char last_s_repp[ECI_MAX_STRING_SIZE];
+  eci_string last_error_repp;
+  eci_string last_s_repp;
+
+  eci_string buffer_rep;
 
   int msgsize_rep;
   int loglevel_rep;
@@ -160,7 +172,7 @@ struct eci_parser {
   int token_phase_rep;
   int buffer_current_rep;
 
-  char buffer_repp[ECI_MAX_PARSER_BUF_SIZE];
+  bool sync_lost_rep;
 };
 
 struct eci_internal { 
@@ -175,7 +187,7 @@ struct eci_internal {
   struct eci_parser* parser_repp;
 
   char farg_buf_repp[ECI_MAX_FLOAT_BUF_SIZE];
-  char raw_buffer_repp[ECI_MAX_PARSER_BUF_SIZE];
+  char raw_buffer_repp[ECI_PARSER_BUF_SIZE];
 };
 
 /* --------------------------------------------------------------------- 
@@ -224,18 +236,19 @@ const char* eci_str_sync_lost =
  * Declarations of static functions
  */
 
-void eci_impl_check_handle(struct eci_internal* eci_rep);
-void eci_impl_clean_last_values(struct eci_parser* parser);
-void eci_impl_dump_parser_state(eci_handle_t ptr, const char* message);
-ssize_t eci_impl_fd_read(int fd, void *buf, size_t count, int timeout);
-const char* eci_impl_get_ecasound_path(void);
-struct eci_los_list *eci_impl_los_list_add_item(struct eci_los_list* headptr, char* stmp, int len);
-struct eci_los_list *eci_impl_los_list_alloc_item(void);
-void eci_impl_los_list_clear(struct eci_los_list *ptr);
-void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout);
-void eci_impl_set_last_los_value(struct eci_parser* parser);
-void eci_impl_set_last_values(struct eci_parser* parser);
-void eci_impl_update_state(struct eci_parser* eci_rep, char c);
+static void eci_impl_check_handle(struct eci_internal* eci_rep);
+static void eci_impl_free_parser(struct eci_internal* eci_rep);
+static void eci_impl_clean_last_values(struct eci_parser* parser);
+static void eci_impl_dump_parser_state(eci_handle_t ptr, const char* message);
+static ssize_t eci_impl_fd_read(int fd, void *buf, size_t count, int timeout);
+static const char* eci_impl_get_ecasound_path(void);
+static struct eci_los_list *eci_impl_los_list_add_item(struct eci_los_list* headptr, char* stmp, int len);
+static struct eci_los_list *eci_impl_los_list_alloc_item(void);
+static void eci_impl_los_list_clear(struct eci_los_list *ptr);
+static void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout);
+static void eci_impl_set_last_los_value(struct eci_parser* parser);
+static void eci_impl_set_last_values(struct eci_parser* parser);
+static void eci_impl_update_state(struct eci_parser* eci_rep, char c);
 
 /* ---------------------------------------------------------------------
  * Constructing and destructing                                       
@@ -255,6 +268,8 @@ void eci_init(void)
 /**
  * Initializes session. This call creates a new ecasound
  * instance and prepares it for processing. 
+ *
+ * @return NULL if initialization fails
  */
 eci_handle_t eci_init_r(void)
 {
@@ -313,7 +328,7 @@ eci_handle_t eci_init_r(void)
       res = write(1, args, 1); 
 
       res = execvp(args[0], (char**)args);
-      if (res < 0) printf("(ecasoundc_sa) launcing ecasound FAILED!\n");
+      if (res < 0) printf("(ecasoundc_sa) launching ecasound FAILED!\n");
 
       close(0);
       close(1);
@@ -328,6 +343,15 @@ eci_handle_t eci_init_r(void)
       int status;
       int pid;
 
+      /* set up signal handling */
+      struct sigaction ign_handler;
+      ign_handler.sa_handler = SIG_IGN;
+      sigemptyset(&ign_handler.sa_mask);
+      ign_handler.sa_flags = 0;
+      /* ignore the following signals */
+      sigaction(SIGPIPE, &ign_handler, 0);
+      sigaction(SIGFPE, &ign_handler, 0);
+
       eci_rep = (struct eci_internal*)calloc(1, sizeof(struct eci_internal));
       eci_rep->parser_repp = (struct eci_parser*)calloc(1, sizeof(struct eci_parser));
 
@@ -337,6 +361,7 @@ eci_handle_t eci_init_r(void)
       eci_rep->parser_repp->last_counter_rep = 0;
       eci_rep->parser_repp->token_phase_rep = ECI_TOKEN_PHASE_NONE;
       eci_rep->parser_repp->buffer_current_rep = 0;
+      eci_rep->parser_repp->sync_lost_rep = false;
       eci_impl_clean_last_values(eci_rep->parser_repp);
 
       /*
@@ -347,7 +372,7 @@ eci_handle_t eci_init_r(void)
       res = read(cmd_receive_pipe[0], &pid, sizeof(pid));
       if ( res != sizeof(pid) ) {
 	  ECI_DEBUG_1("(ecasoundc_sa) fork() of %s FAILED!\n", ecasound_exec);
-	  free(eci_rep->parser_repp);
+	  eci_impl_free_parser(eci_rep);
 	  free(eci_rep);
 	  eci_rep = NULL;
       }
@@ -361,12 +386,13 @@ eci_handle_t eci_init_r(void)
 
       /* step: switch to non-blocking mode for read */
       fcntl(eci_rep->cmd_read_fd_rep, F_SETFL, O_NONBLOCK);
+      fcntl(eci_rep->cmd_write_fd_rep, F_SETFL, O_NONBLOCK);
 
       /* step: check that fork succeeded() */
       res = eci_impl_fd_read(eci_rep->cmd_read_fd_rep, buf, 1, ECI_READ_TIMEOUT_MS);
       if (res != 1) {
 	ECI_DEBUG_1("(ecasoundc_sa) fork() of %s FAILED!\n", ecasound_exec);
-	free(eci_rep->parser_repp);
+	eci_impl_free_parser(eci_rep);
 	free(eci_rep);
 	eci_rep = NULL;
       }
@@ -380,7 +406,7 @@ eci_handle_t eci_init_r(void)
 	eci_impl_read_return_value(eci_rep, ECI_READ_TIMEOUT_MS);
 	if (eci_rep->commands_counter_rep != eci_rep->parser_repp->last_counter_rep) {
 	  ECI_DEBUG_3("(ecasoundc_sa) exec() of %s FAILED (%d=%d)!\n", ecasound_exec, eci_rep->commands_counter_rep, eci_rep->parser_repp->last_counter_rep);
-	  free(eci_rep->parser_repp);
+	  eci_impl_free_parser(eci_rep);
 	  free(eci_rep);
 	  eci_rep = NULL;
 	}
@@ -391,6 +417,36 @@ eci_handle_t eci_init_r(void)
   return (eci_handle_t)eci_rep;
 }
 
+/**
+ * Checks whether ECI is ready for use.
+ *
+ * @return non-zero if ready, zero otherwise
+ */
+int eci_ready(void)
+{
+  return eci_ready_r(static_eci_rep);
+}
+
+/**
+ * Checks whether ECI is ready for use.
+ *
+ * @return non-zero if ready, zero otherwise
+ */
+int eci_ready_r(eci_handle_t ptr)
+{
+  struct eci_internal* eci_rep = (struct eci_internal*)ptr;
+
+  if (!ptr)
+    return 0;
+
+  if (eci_rep->pid_of_child_rep <= 0 ||
+      eci_rep->cmd_read_fd_rep < 0 ||
+      eci_rep->cmd_write_fd_rep < 0)
+    return 0;
+      
+  return 1;
+}
+
 /**
  * Frees all resources.
  */
@@ -410,10 +466,12 @@ void eci_cleanup_r(eci_handle_t ptr)
   struct eci_internal* eci_rep = (struct eci_internal*)ptr;
   ssize_t resread = 1, respoll;
   char buf[1];
-  struct pollfd fds[1];;
+  struct pollfd fds[1];
 
   eci_impl_check_handle(eci_rep);
 
+  ECI_DEBUG("\n(ecasoundc_sa) requesting to terminatte ecasound process.\n");
+
   write(eci_rep->cmd_write_fd_rep, "quit\n", strlen("quit\n"));
   eci_rep->commands_counter_rep++;
   
@@ -448,9 +506,7 @@ void eci_cleanup_r(eci_handle_t ptr)
     /* free lists of strings, if any */
     eci_impl_clean_last_values(eci_rep->parser_repp);
 
-    /* if (eci_rep->eci != 0) {} */
-    /* delete eci_rep->eci; */
-    free(eci_rep->parser_repp);
+    eci_impl_free_parser(eci_rep);
     free(eci_rep);
   }
 }
@@ -476,7 +532,12 @@ void eci_command_r(eci_handle_t ptr, const char* command)
 
   eci_impl_check_handle(eci_rep);
 
-  ECI_DEBUG_2("\n(ecasoundc_sa) writing command '%s' (cmd-counter=%d).\n", 
+  if (eci_ready_r(ptr) == 0) {
+    ECI_DEBUG("(ecasoundc_sa) not ready, unable to process commands\n");
+    return;
+  }
+
+  ECI_DEBUG_2("(ecasoundc_sa) writing command '%s' (cmd-counter=%d).\n", 
 	      command, eci_rep->commands_counter_rep + 1);
 
   memcpy(eci_rep->last_command_repp, command, ECI_MAX_LAST_COMMAND_SIZE);
@@ -488,7 +549,7 @@ void eci_command_r(eci_handle_t ptr, const char* command)
 
   /* 'run' is the only blocking function */
   if (strncmp(command, "run", 3) == 0) {
-    ECI_DEBUG("\n(ecasoundc_sa) 'run' detected; disabling reply timeout!\n");
+    ECI_DEBUG("(ecasoundc_sa) 'run' detected; disabling reply timeout!\n");
     timeout = -1;
   }
 
@@ -497,6 +558,7 @@ void eci_command_r(eci_handle_t ptr, const char* command)
   if (eci_rep->commands_counter_rep - 1 !=
       eci_rep->parser_repp->last_counter_rep) {
     eci_impl_dump_parser_state(ptr, "sync error");
+    eci_rep->parser_repp->sync_lost_rep = true;
   }
   
   if (eci_rep->commands_counter_rep >=
@@ -504,12 +566,13 @@ void eci_command_r(eci_handle_t ptr, const char* command)
     eci_impl_read_return_value(eci_rep, timeout);
   }
 
-  ECI_DEBUG_2("\n(ecasoundc_sa) set return value type='%s' (read-counter=%d).\n", 
+  ECI_DEBUG_2("(ecasoundc_sa) set return value type='%s' (read-counter=%d).\n", 
 	      eci_rep->parser_repp->last_type_repp, eci_rep->parser_repp->last_counter_rep);
   
   if (eci_rep->commands_counter_rep >
       eci_rep->parser_repp->last_counter_rep) {
     fprintf(stderr, "%s", eci_str_sync_lost);
+    eci_rep->parser_repp->sync_lost_rep = true;
   }
 }
 
@@ -592,7 +655,7 @@ const char* eci_last_string_list_item_r(eci_handle_t ptr, int n)
       i != NULL; 
       i = i->next_repp) {
     if (count++ == n) {
-      return i->data_repp;
+      return i->data_repp.d;
     }
   }
 
@@ -607,7 +670,7 @@ const char* eci_last_string_r(eci_handle_t ptr)
 
   eci_impl_check_handle(eci_rep);
 
-  return eci_rep->parser_repp->last_s_repp;
+  return eci_rep->parser_repp->last_s_repp.d;
 }
 
 double eci_last_float(void) { return(eci_last_float_r(static_eci_rep)); }
@@ -659,7 +722,7 @@ const char* eci_last_error_r(eci_handle_t ptr)
 
   eci_impl_check_handle(eci_rep);
   
-  return eci_rep->parser_repp->last_error_repp;
+  return eci_rep->parser_repp->last_error_repp.d;
 }
 
 
@@ -693,8 +756,20 @@ int eci_error_r(eci_handle_t ptr)
 
   eci_impl_check_handle(eci_rep);
 
+  if (eci_ready_r(ptr) == 0) {
+    ECI_DEBUG("(ecasoundc_sa) not ready, raising an error\n");
+    return 1;
+  }
+
+  if (eci_rep->parser_repp->sync_lost_rep == true) {
+    ECI_DEBUG("(ecasoundc_sa) sync lost, raising an error\n");
+    return 1;
+  }
+
   res = (eci_rep->parser_repp->last_type_repp[0] == 'e') ? 1 : 0;
 
+  ECI_DEBUG_1("(ecasoundc_sa) checking for error, returning %d", res);
+
   return res;
 }
  
@@ -713,7 +788,86 @@ const char* eci_current_event_r(eci_handle_t ptr) { return(0); }
  * Implementation of static functions
  */
 
-void eci_impl_check_handle(struct eci_internal* eci_rep)
+static void eci_string_add(eci_string *dst, int at, char const *src, int len);
+
+/**
+ * Clears the string contents.
+ *
+ * @post eci_string_len(str)==0
+ * @post strlen(str->d)==0
+ */
+static void eci_string_clear(eci_string *str)
+{
+  DBC_CHECK(str);
+  str->slen = 0;
+  if (str->size == 0) 
+    eci_string_add(str, 0, NULL, 0);
+  else
+    str->d[0] = 0;
+  DBC_CHECK(str->d[0] == 0);
+}
+
+/**
+ * Initializes the string object for use.
+ * This must only be called after initial 
+ * object allocation.
+ */
+static void eci_string_init(eci_string *str)
+{
+  DBC_CHECK(str);
+  str->slen = 0;
+  str->size = 0;
+  str->d = 0;
+}
+
+static void eci_string_free(eci_string *str)
+{
+  DBC_CHECK(str);
+  free(str->d);
+  str->size = 0;
+  str->slen = 0;
+}
+
+/**
+ * Returns the string length.
+ */
+static int eci_string_len(eci_string *str)
+{
+  DBC_CHECK(str);
+  return str->slen;
+}
+
+/**
+ * Adds 'len' octets from buffer 'src' to the string
+ * at position 'at' (position 0 being the first character).
+ */
+static void eci_string_add(eci_string *dst, int at, char const *src, int len)
+{
+  int space_needed = at + len + 1;
+
+  DBC_CHECK(dst);
+
+  if (space_needed > dst->size) {
+    int newsize = 
+      dst->size ? dst->size * 2 : ECI_INI_STRING_SIZE;
+    char *newbuf;
+    while (space_needed > newsize) {
+      newsize *= 2;
+    }
+    assert(newsize <= ECI_MAX_DYN_ALLOC_SIZE);
+    newbuf = realloc(dst->d, newsize);
+    assert(newbuf);
+    dst->size = newsize;
+    dst->d = newbuf;
+  }
+  
+  DBC_CHECK(space_needed <= dst->size);
+
+  memcpy(&dst->d[at], src, len);
+  dst->d[at + len] = 0;
+}
+
+static void eci_impl_check_handle(struct eci_internal* eci_rep)
 {
   if (eci_rep == NULL) {
     fprintf(stderr, "%s", eci_str_null_handle);
@@ -722,20 +876,30 @@ void eci_impl_check_handle(struct eci_internal* eci_rep)
   }
 }
 
-void eci_impl_clean_last_values(struct eci_parser* parser)
+static void eci_impl_free_parser(struct eci_internal* eci_rep)
+{
+  DBC_CHECK(eci_rep);
+  eci_string_free(&eci_rep->parser_repp->last_error_repp);
+  eci_string_free(&eci_rep->parser_repp->last_s_repp);
+  eci_string_free(&eci_rep->parser_repp->buffer_rep);
+  free(eci_rep->parser_repp);
+  eci_rep->parser_repp = 0;
+}
+
+static void eci_impl_clean_last_values(struct eci_parser* parser)
 {
   DBC_CHECK(parser != 0);
 
-  memset(parser->last_s_repp, 0, ECI_MAX_STRING_SIZE);
   eci_impl_los_list_clear(parser->last_los_repp);
   parser->last_los_repp = NULL;
   parser->last_i_rep = 0;
   parser->last_li_rep = 0;
   parser->last_f_rep = 0.0f;
-  memset(parser->last_error_repp, 0, ECI_MAX_STRING_SIZE);
+  eci_string_clear(&parser->last_error_repp);
+  eci_string_clear(&parser->last_s_repp);
 }
 
-void eci_impl_dump_parser_state(eci_handle_t ptr, const char* message)
+static void eci_impl_dump_parser_state(eci_handle_t ptr, const char* message)
 {
   struct eci_internal* eci_rep = (struct eci_internal*)ptr;
 
@@ -754,7 +918,7 @@ void eci_impl_dump_parser_state(eci_handle_t ptr, const char* message)
  * for reading, up to 'timeout' milliseconds will be waited. 
  * A negative value means infinite timeout.
  */
-ssize_t eci_impl_fd_read(int fd, void *buf, size_t count, int timeout)
+static ssize_t eci_impl_fd_read(int fd, void *buf, size_t count, int timeout)
 {
   int nfds = 1;
   struct pollfd ufds;
@@ -779,7 +943,7 @@ ssize_t eci_impl_fd_read(int fd, void *buf, size_t count, int timeout)
   return rescount;
 }
 
-const char* eci_impl_get_ecasound_path(void)
+static const char* eci_impl_get_ecasound_path(void)
 {
   const char* result = getenv("ECASOUND");
 
@@ -792,16 +956,11 @@ const char* eci_impl_get_ecasound_path(void)
 }
 
 
-struct eci_los_list *eci_impl_los_list_add_item(struct eci_los_list* head, char* stmp, int len)
+static struct eci_los_list *eci_impl_los_list_add_item(struct eci_los_list* head, char* stmp, int len)
 {
   struct eci_los_list* i = head;
   struct eci_los_list* last = NULL;
   
-  if (len + 1 >= ECI_MAX_STRING_SIZE) {
-    fprintf(stderr, "(ecasoundc_sa) WARNING! String list buffer overflowed!\n\n");
-    len = ECI_MAX_STRING_SIZE - 1;
-  }
-
   /* find end of list */
   while(i != NULL) {
     last = i;
@@ -810,8 +969,7 @@ struct eci_los_list *eci_impl_los_list_add_item(struct eci_los_list* head, char*
 
   /* add to the end, copy data */
   i = eci_impl_los_list_alloc_item();
-  memcpy(i->data_repp, stmp, len);
-  i->data_repp[len] = 0;
+  eci_string_add(&i->data_repp, 0, stmp, len);
   if (last != NULL) last->next_repp = i;
   
   /* ECI_DEBUG_3("(ecasoundc_sa) adding item '%s' to los list; head=%p, i=%p\n", stmp, (void*)head, (void*)i); */
@@ -828,16 +986,15 @@ struct eci_los_list *eci_impl_los_list_alloc_item(void)
 {
   struct eci_los_list *item;
   /* ECI_DEBUG("(ecasoundc_sa) list alloc item\n"); */
-  item = (struct eci_los_list*)malloc(sizeof(struct eci_los_list));
+  item = (struct eci_los_list*)calloc(1, sizeof(struct eci_los_list));
   DBC_CHECK(item != NULL);
   item->next_repp = item->prev_repp = NULL;
-  item->data_repp = (char*)malloc(ECI_MAX_STRING_SIZE);
-  DBC_CHECK(item->data_repp != NULL);
+  eci_string_clear(&item->data_repp);
 
   return item;
 }
 
-void eci_impl_los_list_clear(struct eci_los_list *ptr)
+static void eci_impl_los_list_clear(struct eci_los_list *ptr)
 {
   struct eci_los_list *i = ptr;
 
@@ -846,15 +1003,13 @@ void eci_impl_los_list_clear(struct eci_los_list *ptr)
   while(i != NULL) {
     /* ECI_DEBUG_1("(ecasoundc_sa) freeing list item %p\n", (void*)i); */
     struct eci_los_list* next = i->next_repp;
-    if (i->data_repp) {
-      free(i->data_repp), i->data_repp = NULL;
-    }
+    eci_string_free(&i->data_repp);
     free(i);
     i = next;
   }
 }
 
-void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout)
+static void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout)
 {
   char* raw_buffer = eci_rep->raw_buffer_repp;
   int attempts = 0;
@@ -863,7 +1018,7 @@ void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout)
 	    eci_rep->parser_repp->last_counter_rep);
 
   while(attempts < ECI_MAX_RESYNC_ATTEMPTS) {
-    int res = eci_impl_fd_read(eci_rep->cmd_read_fd_rep, raw_buffer, ECI_MAX_PARSER_BUF_SIZE-1, timeout);
+    int res = eci_impl_fd_read(eci_rep->cmd_read_fd_rep, raw_buffer, ECI_PARSER_BUF_SIZE-1, timeout);
     if (res > 0) {
       int n;
 
@@ -884,6 +1039,7 @@ void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout)
     else {
       if (res < 0) {
 	ECI_DEBUG_1("(ecasoundc_sa) timeout when reading return values (attempts=%d)!\n", attempts);
+	eci_rep->parser_repp->sync_lost_rep = true;
 	break;
       }
     }
@@ -893,6 +1049,7 @@ void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout)
   if (eci_rep->commands_counter_rep !=
       eci_rep->parser_repp->last_counter_rep) {
     eci_impl_dump_parser_state(eci_rep, "read() error");
+    eci_rep->parser_repp->sync_lost_rep = true;
   }
 }
 
@@ -902,49 +1059,47 @@ void eci_impl_read_return_value(struct eci_internal* eci_rep, int timeout)
  * @pre parser != 0
  * @pre parser->state_rep == ECI_STATE_COMMON_LF_3
  */
-void eci_impl_set_last_los_value(struct eci_parser* parser)
+static void eci_impl_set_last_los_value(struct eci_parser* parser)
 {
   struct eci_los_list* i = parser->last_los_repp;
   int quoteflag = 0, m = 0, n;
-  char* stmp = malloc(ECI_MAX_STRING_SIZE);
+  eci_string stmp;
+  eci_string_init(&stmp);
 
-  DBC_CHECK(stmp != NULL);
   DBC_CHECK(parser != 0);
   DBC_CHECK(parser->state_rep == ECI_STATE_COMMON_LF_3);
 
-  ECI_DEBUG_2("(ecasoundc_sa) parsing a list '%s' (count=%d)\n", parser->buffer_repp, parser->buffer_current_rep);
+  ECI_DEBUG_2("(ecasoundc_sa) parsing a list '%s' (count=%d)\n", parser->buffer_rep.d, parser->buffer_current_rep);
 
   eci_impl_los_list_clear(i);
   parser->last_los_repp = NULL;
 
   for(n = 0; n < parser->buffer_current_rep && n < parser->msgsize_rep; n++) {
-    char c = parser->buffer_repp[n];
+    char c = parser->buffer_rep.d[n];
 
     if (c == '\"') {
       quoteflag = !quoteflag;
     }
     else if (c == '\\') {
       n++;
-      c = parser->buffer_repp[n];
-      stmp[m++] = c;
+      eci_string_add(&stmp, m++, &parser->buffer_rep.d[n], 1);
     }
     else if (c != ',' || quoteflag == 1) {
-      stmp[m++] = c;
+      eci_string_add(&stmp, m++, &parser->buffer_rep.d[n], 1);
     }
     else {
       if (m == 0) continue;
-      i = eci_impl_los_list_add_item(i, stmp, m);
+      i = eci_impl_los_list_add_item(i, stmp.d, m);
       m = 0;
     }
   }
   if (m > 0) {
-    i = eci_impl_los_list_add_item(i, stmp, m);
+    i = eci_impl_los_list_add_item(i, stmp.d, m);
   }
 
   parser->last_los_repp = i;
 
-  /* delete tmp char-buffer */
-  free(stmp);
+  eci_string_free(&stmp);
 }
 
 /**
@@ -954,7 +1109,7 @@ void eci_impl_set_last_los_value(struct eci_parser* parser)
  * @pre parser != 0
  * @pre parser->state_rep == ECI_STATE_COMMON_LF_3
  */
-void eci_impl_set_last_values(struct eci_parser* parser)
+static void eci_impl_set_last_values(struct eci_parser* parser)
 {
   DBC_CHECK(parser != 0);
   DBC_CHECK(parser->state_rep == ECI_STATE_COMMON_LF_3);
@@ -962,7 +1117,7 @@ void eci_impl_set_last_values(struct eci_parser* parser)
   switch(parser->last_type_repp[0])
     {
     case 's':
-      memcpy(parser->last_s_repp, parser->buffer_repp, parser->buffer_current_rep);
+      eci_string_add(&parser->last_s_repp, 0, parser->buffer_rep.d, parser->buffer_current_rep);
       break;
 
     case 'S':
@@ -970,19 +1125,19 @@ void eci_impl_set_last_values(struct eci_parser* parser)
       break;
 
     case 'i':
-      parser->last_i_rep = atoi(parser->buffer_repp);
+      parser->last_i_rep = atoi(parser->buffer_rep.d);
       break;
 
     case 'l':
-      parser->last_li_rep = atol(parser->buffer_repp);
+      parser->last_li_rep = atol(parser->buffer_rep.d);
       break;
 
     case 'f':
-      parser->last_f_rep = atof(parser->buffer_repp);
+      parser->last_f_rep = atof(parser->buffer_rep.d);
       break;
 
     case 'e':
-      memcpy(parser->last_error_repp, parser->buffer_repp, parser->buffer_current_rep);
+      eci_string_add(&parser->last_error_repp, 0, parser->buffer_rep.d, parser->buffer_current_rep);
       break;
 
     default: {}
@@ -990,16 +1145,15 @@ void eci_impl_set_last_values(struct eci_parser* parser)
     }
 }
 
-void eci_impl_update_state(struct eci_parser* parser, char c)
+static void eci_impl_update_state(struct eci_parser* parser, char c)
 {
-  char* msg_buffer = parser->buffer_repp;
-
   switch(parser->state_rep)
     {
     case ECI_STATE_INIT:
       if (c >= 0x30 && c <= 0x39) {
 	parser->token_phase_rep = ECI_TOKEN_PHASE_READING;
 	parser->buffer_current_rep = 0;
+	eci_string_clear(&parser->buffer_rep);
 	parser->state_rep = ECI_STATE_LOGLEVEL;
       }
       else {
@@ -1009,8 +1163,7 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
 
     case ECI_STATE_LOGLEVEL:
       if (c == ' ') {
-	parser->buffer_repp[parser->buffer_current_rep] = 0;
-	parser->loglevel_rep = atoi(parser->buffer_repp);
+	parser->loglevel_rep = atoi(parser->buffer_rep.d);
 
 	if (parser->loglevel_rep == ECI_RETURN_TYPE_LOGLEVEL) {
 	  /* ECI_DEBUG_3("\n(ecasoundc_sa) found rettype loglevel '%s' (i=%d,len=%d).\n", parser->buffer_repp, parser->loglevel_rep, parser->buffer_current_rep); */
@@ -1034,10 +1187,10 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
       if ((c == ' ' && parser->state_msg_rep == ECI_STATE_MSG_RETURN) ||
 	  (c == 0x0d && parser->state_msg_rep == ECI_STATE_MSG_GEN)) {
 
-	parser->buffer_repp[parser->buffer_current_rep] = 0;
-	parser->msgsize_rep = atoi(parser->buffer_repp);
+	parser->msgsize_rep = atoi(parser->buffer_rep.d);
 
 	/* ECI_DEBUG_3("(ecasoundc_sa) found msgsize '%s' (i=%d,len=%d).\n", parser->buffer_repp, parser->msgsize_rep, parser->buffer_current_rep); */
+
 	if (parser->state_msg_rep == ECI_STATE_MSG_GEN) {
 	  parser->state_rep = ECI_STATE_COMMON_LF_1;
 	}
@@ -1054,6 +1207,7 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
       else if (parser->token_phase_rep == ECI_TOKEN_PHASE_NONE) {
 	parser->token_phase_rep =  ECI_TOKEN_PHASE_READING;
 	parser->buffer_current_rep = 0;
+	eci_string_clear(&parser->buffer_rep);
       }
       break;
 
@@ -1077,11 +1231,11 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
 	/* parse return type */
 	/* set 'parser->last_type_repp' */
 	int len = (parser->buffer_current_rep < ECI_MAX_RETURN_TYPE_SIZE) ? parser->buffer_current_rep : (ECI_MAX_RETURN_TYPE_SIZE - 1);
-	parser->buffer_repp[parser->buffer_current_rep] = 0;
-	memcpy(parser->last_type_repp, parser->buffer_repp, len);
+
+	memcpy(parser->last_type_repp, parser->buffer_rep.d, len);
 	parser->last_type_repp[len] = 0;
 	
-	/* ECI_DEBUG_2("(ecasoundc_sa) found rettype '%s' (len=%d).\n", parser->last_type_repp, parser->buffer_current_rep); */
+	ECI_DEBUG_2("(ecasoundc_sa) found rettype '%s' (len=%d)\n", parser->last_type_repp, parser->buffer_current_rep);
 
 	parser->state_rep = ECI_STATE_COMMON_LF_1;
 	parser->token_phase_rep =  ECI_TOKEN_PHASE_NONE;
@@ -1090,6 +1244,7 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
       else if (parser->token_phase_rep == ECI_TOKEN_PHASE_NONE) {
 	parser->token_phase_rep =  ECI_TOKEN_PHASE_READING;
 	parser->buffer_current_rep = 0;
+	eci_string_clear(&parser->buffer_rep);
       }
 
       break;
@@ -1099,18 +1254,15 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
 	/* parse return type */
 	/* set 'parser->last_xxx_yyy' */
 
-	parser->buffer_repp[parser->buffer_current_rep] = 0;
-
 	/* handle empty content */
-	if (parser->msgsize_rep == 0) parser->buffer_repp[0] = 0;
+	if (parser->msgsize_rep == 0) 
+	  eci_string_clear(&parser->buffer_rep);
 
-#if 1
 	ECI_DEBUG_2("(ecasoundc_sa) found content, loglevel=%d, msgsize=%d", parser->loglevel_rep, parser->msgsize_rep);
 	if (parser->state_msg_rep == ECI_STATE_MSG_GEN)
 	  ECI_DEBUG(".\n");
 	else
 	  ECI_DEBUG_1(" type='%s'.\n", parser->last_type_repp);
-#endif
 
 	parser->state_rep = ECI_STATE_COMMON_LF_2;
 	parser->token_phase_rep =  ECI_TOKEN_PHASE_VALIDATE;
@@ -1119,10 +1271,10 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
       else if (parser->token_phase_rep == ECI_TOKEN_PHASE_NONE) {
 	parser->token_phase_rep = ECI_TOKEN_PHASE_READING;
 	parser->buffer_current_rep = 0;
+	eci_string_clear(&parser->buffer_rep);
       }
       break;
 
-
     case ECI_STATE_COMMON_CR_2:
       if (c == 0x0d)
 	parser->state_rep = ECI_STATE_COMMON_LF_2; 
@@ -1147,12 +1299,12 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
     case ECI_STATE_COMMON_LF_3:
       if (c == 0x0a) {
 	if (parser->state_msg_rep == ECI_STATE_MSG_RETURN) {
-	  ECI_DEBUG_1("(ecasoundc_sa) rettype-content validated: <<< %s >>>\n", parser->buffer_repp);
+	  ECI_DEBUG_1("(ecasoundc_sa) rettype-content validated: <<<%s>>>\n", parser->buffer_rep.d);
 	  eci_impl_set_last_values(parser);
 	  parser->last_counter_rep++;
 	}
 	else {
-	  ECI_DEBUG_1("(ecasoundc_sa) gen-content validated: <<< %s >>>\n", parser->buffer_repp);
+	  ECI_DEBUG_1("(ecasoundc_sa) gen-content validated: <<<%s>>>\n", parser->buffer_rep.d);
 	}
 	parser->state_rep = ECI_STATE_INIT; 
       }
@@ -1172,14 +1324,9 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
     } /* end of switch() */
 
   if (parser->token_phase_rep == ECI_TOKEN_PHASE_READING) {
-    if (parser->buffer_current_rep >= parser->msgsize_rep) {
-      /* ECI_DEBUG_2("\n(ecasoundc_sa) WARNING! Content size exceeded; cur=%d, msgsize=%d.\n\n", parser->buffer_current_rep, parser->msgsize_rep); */
-    }
-    msg_buffer[parser->buffer_current_rep] = c;
-    if (++parser->buffer_current_rep == ECI_MAX_PARSER_BUF_SIZE) {
-      fprintf(stderr, "\n(ecasoundc_sa) WARNING! Parsing buffer overflowed!\n\n");
-      parser->token_phase_rep =  ECI_TOKEN_PHASE_READING;
-      parser->buffer_current_rep = 0;
-    }
+    eci_string_add(&parser->buffer_rep, parser->buffer_current_rep, &c, 1);
+    ++parser->buffer_current_rep;
   }
+
+  //ECI_DEBUG_2("(ecasoundc_sa) parser buf contents: '%s' (cur=%d)\n.", parser->buffer_rep.d, parser->buffer_current_rep);
 }
